# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "policy exemption create",
)
class Create(AAZCommand):
    """Create a policy exemption.

    Create a policy exemption with the given name and scope. Policy exemptions apply to all resources contained within their scope. For example, when you create a policy exemption at resource group scope for a policy assignment at the same or higher scope level, the exemption exempts all applicable resources in the resource group from applying to that policy assignment.

    :example: Create a policy exemption in default subscription
        az policy exemption create -n exemptTestVM --policy-assignment "/subscriptions/{subscriptionId}/providers/Microsoft.Authorization/policyAssignments/limitVMSku" --exemption-category "Waiver"

    :example: Create a policy exemption in the resource group
        az policy exemption create -n exemptTestVM --policy-assignment "/subscriptions/{subscriptionId}/providers/Microsoft.Authorization/policyAssignments/limitVMSku" --exemption-category "Waiver" --resource-group "myResourceGroup"

    :example: Create a policy exemption in a management group
        az policy exemption create -n exemptTestVM --policy-assignment "/providers/Microsoft.Management/managementGroups/{managementGroupName}/providers/Microsoft.Authorization/policyAssignments/limitVMSku" --exemption-category "Waiver" --scope "/providers/Microsoft.Management/managementGroups/{managementGroupName}"
    """

    _aaz_info = {
        "version": "2026-01-01-preview",
        "resources": [
            ["mgmt-plane", "/{scope}/providers/microsoft.authorization/policyexemptions/{}", "2026-01-01-preview"],
        ]
    }

    def _handler(self, command_args):
        super()._handler(command_args)
        self._execute_operations()
        return self._output()

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.name = AAZStrArg(
            options=["-n", "--name"],
            help="The name of the policy exemption.",
            required=True,
            fmt=AAZStrArgFormat(
                pattern="^[^<>%&:\\?/]*[^<>%&:\\?/ ]+$",
            ),
        )
        _args_schema.scope = AAZStrArg(
            options=["--scope"],
            help={"short-summary": "The scope of the policy assignment.", "long-summary": "Valid scopes are: management group (format: '/providers/Microsoft.Management/managementGroups/{managementGroup}'), subscription (format: '/subscriptions/{subscriptionId}'), resource group (format: '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}', or resource (format: '/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/{resourceProviderNamespace}/[{parentResourcePath}/]{resourceType}/{resourceName}'. The scope of an assignment is always the part of its ID preceding '/providers/Microsoft.Authorization/policyAssignments/{policyAssignmentName}'. If scope is not provided, the scope will be the implied or specified subscription."},
            required=True,
        )

        # define Arg Group "Properties"

        _args_schema = cls._args_schema
        _args_schema.assignment_scope_validation = AAZStrArg(
            options=["-v", "--assignment-scope-validation"],
            arg_group="Properties",
            help={"short-summary": "The assignment scope validation", "long-summary": "Indicates the type of validation to perform on the assignment scope. Valid values are Default, DoNotValidate."},
            default="Default",
            enum={"Default": "Default", "DoNotValidate": "DoNotValidate"},
        )
        _args_schema.description = AAZStrArg(
            options=["--description"],
            arg_group="Properties",
            help={"short-summary": "Policy exemption description.", "long-summary": "Full description of the policy exemption."},
        )
        _args_schema.display_name = AAZStrArg(
            options=["--display-name"],
            arg_group="Properties",
            help={"short-summary": "The display name of the policy exemption.", "long-summary": "The display name of the policy exemption is not part of its ID, allowing for longer and more flexible naming."},
        )
        _args_schema.exemption_category = AAZStrArg(
            options=["-e", "--exemption-category"],
            arg_group="Properties",
            help={"short-summary": "The policy exemption category.", "long-summary": "The policy exemption category. Possible values are Mitigated and Waiver."},
            enum={"Mitigated": "Mitigated", "Waiver": "Waiver"},
        )
        _args_schema.expires_on = AAZDateTimeArg(
            options=["--expires-on"],
            arg_group="Properties",
            help={"short-summary": "The expiration date and time.", "long-summary": "The expiration date and time of the policy exemption in UTC ISO 8601 format, e.g. yyyy-MM-ddTHH:mm:ssZ."},
            fmt=AAZDateTimeFormat(
                protocol="iso",
            ),
        )
        _args_schema.metadata = AAZAnyTypeArg(
            options=["--metadata"],
            arg_group="Properties",
            help={"short-summary": "The policy exemption metadata.", "long-summary": "The policy exemption metadata. Metadata is an open-ended object and is typically a collection of key value pairs."},
        )
        _args_schema.policy_assignment = AAZStrArg(
            options=["-a", "--policy-assignment"],
            arg_group="Properties",
            help={"short-summary": "The policy assignment to exempt.", "long-summary": "The resource ID of the policy assignment to exempt."},
        )
        _args_schema.policy_definition_reference_ids = AAZListArg(
            options=["-r", "--policy-definition-reference-ids"],
            arg_group="Properties",
            help={"short-summary": "The policy definition reference IDs.", "long-summary": "The collection of policy definition reference IDs to exempt when the policy assignment is an assignment of a policy set definition."},
        )
        _args_schema.resource_selectors = AAZListArg(
            options=["--resource-selectors"],
            arg_group="Properties",
            help={"short-summary": "The resource selectors list to filter policies by resource properties.", "long-summary": "The collection of resource selector expressions used to filter policy exemption applicability by certain resource property values."},
        )

        policy_definition_reference_ids = cls._args_schema.policy_definition_reference_ids
        policy_definition_reference_ids.Element = AAZStrArg()

        resource_selectors = cls._args_schema.resource_selectors
        resource_selectors.Element = AAZObjectArg()

        _element = cls._args_schema.resource_selectors.Element
        _element.name = AAZStrArg(
            options=["name"],
            help="The name of the resource selector.",
        )
        _element.selectors = AAZListArg(
            options=["selectors"],
            help="The list of the selector expressions.",
        )

        selectors = cls._args_schema.resource_selectors.Element.selectors
        selectors.Element = AAZObjectArg()

        _element = cls._args_schema.resource_selectors.Element.selectors.Element
        _element.in_ = AAZListArg(
            options=["in"],
            help="The list of values to filter in.",
        )
        _element.kind = AAZStrArg(
            options=["kind"],
            help="The selector kind.",
            enum={"groupPrincipalId": "groupPrincipalId", "policyDefinitionReferenceId": "policyDefinitionReferenceId", "resourceLocation": "resourceLocation", "resourceType": "resourceType", "resourceWithoutLocation": "resourceWithoutLocation", "userPrincipalId": "userPrincipalId"},
        )
        _element.not_in = AAZListArg(
            options=["not-in"],
            help="The list of values to filter out.",
        )

        in_ = cls._args_schema.resource_selectors.Element.selectors.Element.in_
        in_.Element = AAZStrArg()

        not_in = cls._args_schema.resource_selectors.Element.selectors.Element.not_in
        not_in.Element = AAZStrArg()
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        self.PolicyExemptionsCreateOrUpdate(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class PolicyExemptionsCreateOrUpdate(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200, 201]:
                return self.on_200_201(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/{scope}/providers/Microsoft.Authorization/policyExemptions/{policyExemptionName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "PUT"

        @property
        def error_format(self):
            return "MgmtErrorFormat"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "policyExemptionName", self.ctx.args.name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "scope", self.ctx.args.scope,
                    skip_quote=True,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2026-01-01-preview",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Content-Type", "application/json",
                ),
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        @property
        def content(self):
            _content_value, _builder = self.new_content_builder(
                self.ctx.args,
                typ=AAZObjectType,
                typ_kwargs={"flags": {"required": True, "client_flatten": True}}
            )
            _builder.set_prop("properties", AAZObjectType, typ_kwargs={"flags": {"client_flatten": True}})

            properties = _builder.get(".properties")
            if properties is not None:
                properties.set_prop("assignmentScopeValidation", AAZStrType, ".assignment_scope_validation")
                properties.set_prop("description", AAZStrType, ".description")
                properties.set_prop("displayName", AAZStrType, ".display_name")
                properties.set_prop("exemptionCategory", AAZStrType, ".exemption_category", typ_kwargs={"flags": {"required": True}})
                properties.set_prop("expiresOn", AAZStrType, ".expires_on")
                properties.set_prop("metadata", AAZAnyType, ".metadata")
                properties.set_prop("policyAssignmentId", AAZStrType, ".policy_assignment", typ_kwargs={"flags": {"required": True}})
                properties.set_prop("policyDefinitionReferenceIds", AAZListType, ".policy_definition_reference_ids")
                properties.set_prop("resourceSelectors", AAZListType, ".resource_selectors")

            policy_definition_reference_ids = _builder.get(".properties.policyDefinitionReferenceIds")
            if policy_definition_reference_ids is not None:
                policy_definition_reference_ids.set_elements(AAZStrType, ".")

            resource_selectors = _builder.get(".properties.resourceSelectors")
            if resource_selectors is not None:
                resource_selectors.set_elements(AAZObjectType, ".")

            _elements = _builder.get(".properties.resourceSelectors[]")
            if _elements is not None:
                _elements.set_prop("name", AAZStrType, ".name")
                _elements.set_prop("selectors", AAZListType, ".selectors")

            selectors = _builder.get(".properties.resourceSelectors[].selectors")
            if selectors is not None:
                selectors.set_elements(AAZObjectType, ".")

            _elements = _builder.get(".properties.resourceSelectors[].selectors[]")
            if _elements is not None:
                _elements.set_prop("in", AAZListType, ".in_")
                _elements.set_prop("kind", AAZStrType, ".kind")
                _elements.set_prop("notIn", AAZListType, ".not_in")

            in_ = _builder.get(".properties.resourceSelectors[].selectors[].in")
            if in_ is not None:
                in_.set_elements(AAZStrType, ".")

            not_in = _builder.get(".properties.resourceSelectors[].selectors[].notIn")
            if not_in is not None:
                not_in.set_elements(AAZStrType, ".")

            return self.serialize_content(_content_value)

        def on_200_201(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200_201
            )

        _schema_on_200_201 = None

        @classmethod
        def _build_schema_on_200_201(cls):
            if cls._schema_on_200_201 is not None:
                return cls._schema_on_200_201

            cls._schema_on_200_201 = AAZObjectType()

            _schema_on_200_201 = cls._schema_on_200_201
            _schema_on_200_201.id = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200_201.name = AAZStrType(
                flags={"read_only": True},
            )
            _schema_on_200_201.properties = AAZObjectType(
                flags={"client_flatten": True},
            )
            _schema_on_200_201.system_data = AAZObjectType(
                serialized_name="systemData",
                flags={"read_only": True},
            )
            _schema_on_200_201.type = AAZStrType(
                flags={"read_only": True},
            )

            properties = cls._schema_on_200_201.properties
            properties.assignment_scope_validation = AAZStrType(
                serialized_name="assignmentScopeValidation",
            )
            properties.description = AAZStrType()
            properties.display_name = AAZStrType(
                serialized_name="displayName",
            )
            properties.exemption_category = AAZStrType(
                serialized_name="exemptionCategory",
                flags={"required": True},
            )
            properties.expires_on = AAZStrType(
                serialized_name="expiresOn",
            )
            properties.metadata = AAZAnyType()
            properties.policy_assignment_id = AAZStrType(
                serialized_name="policyAssignmentId",
                flags={"required": True},
            )
            properties.policy_definition_reference_ids = AAZListType(
                serialized_name="policyDefinitionReferenceIds",
            )
            properties.resource_selectors = AAZListType(
                serialized_name="resourceSelectors",
            )

            policy_definition_reference_ids = cls._schema_on_200_201.properties.policy_definition_reference_ids
            policy_definition_reference_ids.Element = AAZStrType()

            resource_selectors = cls._schema_on_200_201.properties.resource_selectors
            resource_selectors.Element = AAZObjectType()

            _element = cls._schema_on_200_201.properties.resource_selectors.Element
            _element.name = AAZStrType()
            _element.selectors = AAZListType()

            selectors = cls._schema_on_200_201.properties.resource_selectors.Element.selectors
            selectors.Element = AAZObjectType()

            _element = cls._schema_on_200_201.properties.resource_selectors.Element.selectors.Element
            _element["in"] = AAZListType()
            _element.kind = AAZStrType()
            _element.not_in = AAZListType(
                serialized_name="notIn",
            )

            in_ = cls._schema_on_200_201.properties.resource_selectors.Element.selectors.Element["in"]
            in_.Element = AAZStrType()

            not_in = cls._schema_on_200_201.properties.resource_selectors.Element.selectors.Element.not_in
            not_in.Element = AAZStrType()

            system_data = cls._schema_on_200_201.system_data
            system_data.created_at = AAZStrType(
                serialized_name="createdAt",
            )
            system_data.created_by = AAZStrType(
                serialized_name="createdBy",
            )
            system_data.created_by_type = AAZStrType(
                serialized_name="createdByType",
            )
            system_data.last_modified_at = AAZStrType(
                serialized_name="lastModifiedAt",
            )
            system_data.last_modified_by = AAZStrType(
                serialized_name="lastModifiedBy",
            )
            system_data.last_modified_by_type = AAZStrType(
                serialized_name="lastModifiedByType",
            )

            return cls._schema_on_200_201


class _CreateHelper:
    """Helper class for Create"""


__all__ = ["Create"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from .__cmd_group import *
from ._create import *
from ._delete import *
from ._show import *
from ._update import *
from ._list import *



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=protected-access
import unittest
from unittest import mock
import os

from azure.cli.core.commands.client_factory import get_mgmt_service_client
from azure.cli.core.mock import DummyCli
from azure.cli.core.profiles import ResourceType
from azure.cli.testsdk import ScenarioTest, LiveScenarioTest
from knack.util import CLIError
from azure.cli.testsdk import live_only, MOCKED_USER_NAME
from azure.cli.testsdk.constants import AUX_SUBSCRIPTION, AUX_TENANT

from azure.cli.testsdk.scenario_tests.const import MOCKED_SUBSCRIPTION_ID, MOCKED_TENANT_ID

mock_subscriptions = [
    {
        "id": MOCKED_SUBSCRIPTION_ID,
        "state": "Enabled",
        "name": "Example",
        "tenantId": MOCKED_TENANT_ID,
        "isDefault": True,
        "user": {
            "name": MOCKED_USER_NAME,
            "type": "user"
        }
    },
    {
        "id": AUX_SUBSCRIPTION,
        "state": "Enabled",
        "name": "Auxiliary Subscription",
        "tenantId": AUX_TENANT,
        "isDefault": False,
        "user": {
            "name": MOCKED_USER_NAME,
            "type": "user"
        }
    }
]


class CredentialMock:
    def __init__(self, *args, **kwargs):
        super().__init__()
        self._authority = kwargs.get('authority')

    def get_token(self, *scopes, **kwargs):  # pylint: disable=unused-argument
        from azure.core.credentials import AccessToken
        import time
        now = int(time.time())
        return AccessToken("access_token_from_" + self._authority, now + 3600)


class TestClientFactory(unittest.TestCase):
    def test_get_mgmt_service_client(self):
        cli = DummyCli()
        client = get_mgmt_service_client(cli, ResourceType.MGMT_RESOURCE_RESOURCES)
        assert client

    @mock.patch("azure.cli.core.auth.identity.UserCredential", CredentialMock)
    @mock.patch('azure.cli.core._profile.Profile.load_cached_subscriptions', return_value=mock_subscriptions)
    def test_get_mgmt_service_client_with_aux_subs_and_tenants(self, load_cached_subscriptions_mock):
        cli = DummyCli()

        def _verify_client_aux_token(client_to_check):
            aux_tokens = client_to_check._config.headers_policy.headers.get('x-ms-authorization-auxiliary')
            assert aux_tokens
            assert aux_tokens.startswith("Bearer ")
            assert AUX_TENANT in aux_tokens

        # Specify aux_subscriptions
        client = get_mgmt_service_client(cli, ResourceType.MGMT_RESOURCE_RESOURCES,
                                         aux_subscriptions=[AUX_SUBSCRIPTION])
        _verify_client_aux_token(client)

        # Specify aux_tenants
        client = get_mgmt_service_client(cli, ResourceType.MGMT_RESOURCE_RESOURCES,
                                         aux_tenants=[AUX_TENANT])
        _verify_client_aux_token(client)

        # But not both
        with self.assertRaisesRegex(CLIError, "only one of aux_subscriptions and aux_tenants"):
            get_mgmt_service_client(cli, ResourceType.MGMT_RESOURCE_RESOURCES,
                                    aux_subscriptions=[AUX_SUBSCRIPTION], aux_tenants=[AUX_TENANT])

    def test_custom_credential(self):
        from azure.cli.core.auth.util import AccessToken

        class AccessTokenCredential:
            """Simple access token authentication. Return the access token as-is.
            """
            def __init__(self, access_token):
                self.access_token = access_token
                self.get_token_called = False

            def get_token(self, *scopes, **kwargs):
                self.get_token_called = True
                import time
                # Assume the access token expires in 1 year
                return AccessToken(self.access_token, int(time.time()) + 31536000)

        cli = DummyCli()
        cred = AccessTokenCredential("mock_token")
        client = get_mgmt_service_client(cli, ResourceType.MGMT_RESOURCE_RESOURCES,
                                         subscription_id=MOCKED_SUBSCRIPTION_ID, credential=cred)
        assert isinstance(client._config.credential, AccessTokenCredential)

        from azure.core.exceptions import ResourceNotFoundError
        try:
            client.resource_groups.get('test_rg')
        except ResourceNotFoundError:
            # Error is expected since we are using a dummy subscription ID
            pass
        assert cred.get_token_called

        # Error if no subscription is provided
        with self.assertRaises(ValueError):
            get_mgmt_service_client(cli, ResourceType.MGMT_RESOURCE_RESOURCES, credential=cred)


if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import unittest
from unittest.mock import Mock

from azure.cli.command_modules.acs._polling import RunCommandLocationPolling
from azure.core.pipeline import PipelineResponse
from azure.core.rest import HttpRequest, HttpResponse


class TestRunCommandPoller(unittest.TestCase):
    def test_get_status(self):
        poller = RunCommandLocationPolling()

        mock_response = Mock(spec=HttpResponse)
        mock_response.text.return_value = "{\"properties\": {\"provisioningState\": \"Scaling Up\"}}"

        pipeline_response: PipelineResponse[HttpRequest, HttpResponse] = PipelineResponse(Mock(spec=HttpRequest), mock_response, Mock())

        status = poller.get_status(pipeline_response)
        assert status == "Scaling Up"

    def test_get_status_no_provisioning_state(self):
        poller = RunCommandLocationPolling()

        mock_response = Mock(spec=HttpResponse)
        mock_response.text.return_value = "{\"properties\": {\"status\": \"Scaling Up\"}}"

        pipeline_response: PipelineResponse[HttpRequest, HttpResponse] = PipelineResponse(Mock(spec=HttpRequest), mock_response, Mock())

        status = poller.get_status(pipeline_response)
        assert status == "Unknown"



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

from azure.cli.testsdk import ScenarioTest


class ComputeRecommenderScenario(ScenarioTest):
    def test_spot_placement_recommender_generate(self):        
        self.kwargs.update({
            'location': 'eastus',
            'subscription_id': self.get_subscription_id(),
            'availability_zones': 'true',
            'desired_locations': '["eastus", "eastus2"]',
            'desired_count': 1,
            'desired_sizes': '[{"sku": "Standard_D2_v2"}]'
        })

        spot_scores_output = self.cmd('az compute-recommender spot-placement-recommender -l {location} --subscription {subscription_id} --availability-zones {availability_zones} --desired-locations \'{desired_locations}\' --desired-count {desired_count} --desired-sizes \'{desired_sizes}\'').get_output_in_json()

        self.assertTrue(len(spot_scores_output["placementScores"]) > 0, "Spot scores should have at least one item")

    def test_spot_placement_score_generate(self):        
        self.kwargs.update({
            'location': 'eastus',
            'subscription_id': self.get_subscription_id(),
            'availability_zones': 'true',
            'desired_locations': '["eastus", "eastus2"]',
            'desired_count': 1,
            'desired_sizes': '[{"sku": "Standard_D2_v2"}]'
        })

        spot_scores_output = self.cmd('az compute-recommender spot-placement-score -l {location} --subscription {subscription_id} --availability-zones {availability_zones} --desired-locations \'{desired_locations}\' --desired-count {desired_count} --desired-sizes \'{desired_sizes}\'').get_output_in_json()

        self.assertTrue(len(spot_scores_output["placementScores"]) > 0, "Spot scores should have at least one item")


# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
import os
import unittest  # pylint: disable=unused-import

from azure.cli.testsdk import (ResourceGroupPreparer)
from azure.cli.testsdk.decorators import serial_test
from azure.cli.command_modules.containerapp.tests.latest.common import (
    ContainerappComposePreviewScenarioTest,  # pylint: disable=unused-import
    write_test_file,
    clean_up_test_file,
    TEST_DIR, TEST_LOCATION)

from .utils import prepare_containerapp_env_for_app_e2e_tests


# flake8: noqa
# noqa
# pylint: skip-file


class ContainerappComposePreviewResourceSettingsScenarioTest(ContainerappComposePreviewScenarioTest):
    def __init__(self, *arg, **kwargs):
        super().__init__(*arg, random_config_dir=True, **kwargs)

    @serial_test()
    @ResourceGroupPreparer(name_prefix='cli_test_containerapp_preview', location='eastus')
    def test_containerapp_compose_create_with_resources_from_service_cpus(self, resource_group):
        self.cmd('configure --defaults location={}'.format(TEST_LOCATION))

        compose_text = """
services:
  foo:
    image: mcr.microsoft.com/azuredocs/aks-helloworld:v1
    cpus: 1.25
    mem_reservation: 2.5g
    expose:
      - "3000"
"""
        compose_file_name = f"{self._testMethodName}_compose.yml"
        write_test_file(compose_file_name, compose_text)
        env_id = prepare_containerapp_env_for_app_e2e_tests(self)

        self.kwargs.update({
            'environment': env_id,
            'compose': compose_file_name,
        })

        command_string = 'containerapp compose create'
        command_string += ' --compose-file-path {compose}'
        command_string += ' --resource-group {rg}'
        command_string += ' --environment {environment}'
        self.cmd(command_string, checks=[
            self.check('[?name==`foo`].properties.template.containers[0].resources.cpu', [1.25]),
        ])
        self.cmd(f'containerapp delete -n foo -g {resource_group} --yes', expect_failure=False)
        clean_up_test_file(compose_file_name)

    @serial_test()
    @ResourceGroupPreparer(name_prefix='cli_test_containerapp_preview', location='eastus')
    def test_containerapp_compose_create_with_resources_from_deploy_cpu(self, resource_group):
        self.cmd('configure --defaults location={}'.format(TEST_LOCATION))

        compose_text = """
services:
  foo:
    image: mcr.microsoft.com/azuredocs/aks-helloworld:v1
    deploy:
      resources:
        reservations:
          cpus: 1.25
          memory: 2.5g
    expose:
      - "3000"
"""
        compose_file_name = f"{self._testMethodName}_compose.yml"
        write_test_file(compose_file_name, compose_text)
        env_id = prepare_containerapp_env_for_app_e2e_tests(self)

        self.kwargs.update({
            'environment': env_id,
            'compose': compose_file_name,
        })

        command_string = 'containerapp compose create'
        command_string += ' --compose-file-path {compose}'
        command_string += ' --resource-group {rg}'
        command_string += ' --environment {environment}'
        self.cmd(command_string, checks=[
            self.check('[?name==`foo`].properties.template.containers[0].resources.cpu', [1.25]),
        ])
        self.cmd(f'containerapp delete -n foo -g {resource_group} --yes', expect_failure=False)
        clean_up_test_file(compose_file_name)

    @serial_test()
    @ResourceGroupPreparer(name_prefix='cli_test_containerapp_preview', location='eastus')
    def test_containerapp_compose_create_with_resources_from_both_cpus_and_deploy_cpu(self, resource_group):
        self.cmd('configure --defaults location={}'.format(TEST_LOCATION))

        compose_text = """
services:
  foo:
    image: mcr.microsoft.com/azuredocs/aks-helloworld:v1
    cpus: 0.75
    deploy:
      resources:
        reservations:
          cpus: 1.25
          memory: 2.5g
    expose:
      - "3000"
"""
        compose_file_name = f"{self._testMethodName}_compose.yml"
        write_test_file(compose_file_name, compose_text)
        env_id = prepare_containerapp_env_for_app_e2e_tests(self)

        self.kwargs.update({
            'environment': env_id,
            'compose': compose_file_name,
        })
        
        command_string = 'containerapp compose create'
        command_string += ' --compose-file-path {compose}'
        command_string += ' --resource-group {rg}'
        command_string += ' --environment {environment}'
        self.cmd(command_string, checks=[
            self.check('[?name==`foo`].properties.template.containers[0].resources.cpu', [1.25]),
        ])
        self.cmd(f'containerapp delete -n foo -g {resource_group} --yes', expect_failure=False)
        clean_up_test_file(compose_file_name)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
import os
import time
import unittest

from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer
from azure.cli.testsdk.scenario_tests import AllowLargeResponse
from knack.util import CLIError


class CognitiveServicesDeleteTests(ScenarioTest):
    @ResourceGroupPreparer()
    @AllowLargeResponse()
    def test_cognitiveservices_softdelete(self, resource_group):
        sname = self.create_random_name(prefix='cs_cli_test_', length=16)
        customdomain = self.create_random_name(prefix='csclitest', length=16)

        self.kwargs.update({
            'sname': sname,
            'kind': 'FormRecognizer',
            'sku': 'S0',
            'location': 'SOUTHCENTRALUS',
            'customdomain': customdomain
        })

        # test to create cognitive services account
        self.cmd('az cognitiveservices account create -n {sname} -g {rg} --kind {kind} --sku {sku} -l {location} --custom-domain {customdomain} --yes',
                 checks=[self.check('name', '{sname}'),
                         self.check('location', '{location}'),
                         self.check('sku.name', '{sku}'),
                         self.check('properties.customSubDomainName', '{customdomain}')])

        # test to show the details of cognitive services account
        self.cmd('az cognitiveservices account show -n {sname} -g {rg}',
                 checks=[self.check('name', '{sname}'),
                         self.check('resourceGroup', '{rg}'),
                         self.check('properties.customSubDomainName', '{customdomain}')])

        # test to update the properties of cognitive services account
        self.cmd('az cognitiveservices account delete -n {sname} -g {rg}')

        self.cmd('az cognitiveservices account show-deleted --location {location} -n {sname} -g {rg}',
                 checks=[self.check('name', '{sname}'),
                         self.check('properties.customSubDomainName', '{customdomain}')])

        self.cmd('az cognitiveservices account recover --location {location} -n {sname} -g {rg}')

        self.cmd('az cognitiveservices account show -n {sname} -g {rg}',
                 checks=[self.check('name', '{sname}'),
                         self.check('resourceGroup', '{rg}'),
                         self.check('properties.customSubDomainName', '{customdomain}')])


        self.cmd('az cognitiveservices account delete -n {sname} -g {rg}')
        self.cmd('az cognitiveservices account purge --location {location} -n {sname} -g {rg}')
        deleted_accounts = self.cmd('az cognitiveservices account list-deleted --query "[?name==`{sname}`]"').get_output_in_json()

        self.assertEqual(len(deleted_accounts), 0)
        

if __name__ == '__main__':
    unittest.main()



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from knack.log import get_logger
from .util import build_sdk_access_token

logger = get_logger(__name__)


class CredentialAdaptor:
    def __init__(self, credential, auxiliary_credentials=None):
        """Credential adaptor between MSAL credential and SDK credential.
        It implements Track 2 SDK's azure.core.credentials.TokenCredential by exposing get_token.

        :param credential: MSAL credential from ._msal_credentials
        :param auxiliary_credentials: MSAL credentials for cross-tenant authentication.
            Details about cross-tenant authentication:
            https://learn.microsoft.com/en-us/azure/azure-resource-manager/management/authenticate-multi-tenant
        """

        self._credential = credential
        self._auxiliary_credentials = auxiliary_credentials

    def get_token(self, *scopes, **kwargs):
        """Implement the old SDK token protocol azure.core.credentials.TokenCredential
        Return azure.core.credentials.AccessToken
        """
        logger.debug("CredentialAdaptor.get_token: scopes=%r, kwargs=%r", scopes, kwargs)

        msal_kwargs = _prepare_msal_kwargs(kwargs)
        msal_result = self._credential.acquire_token(list(scopes), **msal_kwargs)
        return build_sdk_access_token(msal_result)

    def get_token_info(self, *scopes, options=None):
        """Implement the new SDK token protocol azure.core.credentials.SupportsTokenInfo
        Return azure.core.credentials.AccessTokenInfo
        """
        logger.debug("CredentialAdaptor.get_token_info: scopes=%r, options=%r", scopes, options)

        msal_kwargs = _prepare_msal_kwargs(options)
        msal_result = self._credential.acquire_token(list(scopes), **msal_kwargs)
        return _build_sdk_access_token_info(msal_result)

    def get_auxiliary_tokens(self, *scopes, **kwargs):
        """Get access tokens from auxiliary credentials."""
        # To test cross-tenant authentication, see https://github.com/Azure/azure-cli/issues/16691
        if self._auxiliary_credentials:
            return [build_sdk_access_token(cred.acquire_token(list(scopes), **kwargs))
                    for cred in self._auxiliary_credentials]
        return None


def _prepare_msal_kwargs(options=None):
    # Preserve supported options and discard unsupported options (tenant_id, enable_cae).
    # Both get_token's kwargs and get_token_info's options are accepted as their schema is the same (at least for now).
    msal_kwargs = {}
    if options:
        # For VM SSH. 'data' support is a CLI-specific extension.
        # SDK doesn't support 'data': https://github.com/Azure/azure-sdk-for-python/pull/16397
        if 'data' in options:
            msal_kwargs['data'] = options['data']
        # For CAE
        if 'claims' in options:
            msal_kwargs['claims_challenge'] = options['claims']
    return msal_kwargs


def _build_sdk_access_token_info(token_entry):
    # MSAL token entry sample:
    # {
    #     'access_token': 'eyJ0eXAiOiJKV...',
    #     'token_type': 'Bearer',
    #     'expires_in': 1618,
    #     'token_source': 'cache'
    # }
    from .constants import ACCESS_TOKEN, EXPIRES_IN
    from .util import now_timestamp
    from azure.core.credentials import AccessTokenInfo

    return AccessTokenInfo(token_entry[ACCESS_TOKEN], now_timestamp() + token_entry[EXPIRES_IN])



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.core.azclierror import InvalidArgumentValueError
from azure.mgmt.cognitiveservices.models import ConnectionCategory, \
    ResourceIdentityType as IdentityType, Identity, UserAssignedIdentity

from os import PathLike
from typing import IO, Any, AnyStr, Dict, Optional, Union


def compose_identity(
    system_assigned: bool,
    user_assigned_identity: Optional[str]
):
    identity_type = None
    user_identities = None
    if user_assigned_identity is not None:
        user_identities = {user_assigned_identity: UserAssignedIdentity()}

    if system_assigned is not None and user_assigned_identity is not None:
        identity_type = IdentityType.SYSTEM_ASSIGNED_USER_ASSIGNED
    elif system_assigned:
        identity_type = IdentityType.SYSTEM_ASSIGNED
    elif user_assigned_identity is not None:
        identity_type = IdentityType.USER_ASSIGNED

    return Identity(type=identity_type, user_assigned_identities=user_identities) if identity_type is not None else None


def snake_to_camel(name):
    import re
    return re.sub(r"(?:^|_)([a-z])", lambda x: x.group(1).upper(), name)


def get_auth_model_connection_properties(connection_category: str, auth_type: str | None, auth_params: Dict[str, Any]):  # pylint: disable=too-many-locals
    from azure.mgmt.cognitiveservices.models import (
        PATAuthTypeConnectionProperties,
        SASAuthTypeConnectionProperties,
        UsernamePasswordAuthTypeConnectionProperties,
        ManagedIdentityAuthTypeConnectionProperties,
        OAuth2AuthTypeConnectionProperties,
        ServicePrincipalAuthTypeConnectionProperties,
        AccessKeyAuthTypeConnectionProperties,
        ApiKeyAuthConnectionProperties,
        NoneAuthTypeConnectionProperties,
        AccountKeyAuthTypeConnectionProperties,
        AADAuthTypeConnectionProperties,
        ConnectionPersonalAccessToken,
        ConnectionSharedAccessSignature,
        ConnectionUsernamePassword,
        ConnectionAccessKey,
        ConnectionApiKey,
        ConnectionAccountKey,
        ConnectionServicePrincipal,
        ConnectionManagedIdentity,
        ConnectionOAuth2
    )
    # These categories were using None auth type with AzCLI ML connections
    CAN_USE_NONE_AUTH = [ConnectionCategory.GIT, ConnectionCategory.PYTHON_FEED]
    # SDK 15.x renamed some credential params; remap old names from YAML/JSON files
    _CREDENTIAL_PARAM_RENAMES = {
        'account_key': 'key',
        'sas_token': 'sas',
    }
    mapped_params = {_CREDENTIAL_PARAM_RENAMES.get(k, k): v for k, v in auth_params.items()}
    auth_model = None
    match snake_to_camel(auth_type):
        case "PersonalAccessToken" | "Pat":
            auth_model = PATAuthTypeConnectionProperties(
                credentials=ConnectionPersonalAccessToken(**mapped_params))
        case "SasToken" | "Sas":
            auth_model = SASAuthTypeConnectionProperties(
                credentials=ConnectionSharedAccessSignature(**mapped_params))
        case "UsernamePassword":
            auth_model = UsernamePasswordAuthTypeConnectionProperties(
                credentials=ConnectionUsernamePassword(**mapped_params))
        case "ManagedIdentity":
            auth_model = ManagedIdentityAuthTypeConnectionProperties(
                credentials=ConnectionManagedIdentity(**mapped_params))
        case "ServicePrincipal":
            auth_model = ServicePrincipalAuthTypeConnectionProperties(
                credentials=ConnectionServicePrincipal(**mapped_params))
        case "AccessKey":
            auth_model = AccessKeyAuthTypeConnectionProperties(
                credentials=ConnectionAccessKey(**mapped_params))
        case "ApiKey":
            auth_model = ApiKeyAuthConnectionProperties(
                credentials=ConnectionApiKey(**mapped_params))
        case "OAuth2":
            auth_model = OAuth2AuthTypeConnectionProperties(
                credentials=ConnectionOAuth2(**mapped_params))
        case "None":
            if connection_category in CAN_USE_NONE_AUTH:
                auth_model = NoneAuthTypeConnectionProperties()
            else:
                auth_model = AADAuthTypeConnectionProperties()
        case "AccountKey":
            auth_model = AccountKeyAuthTypeConnectionProperties(
                credentials=ConnectionAccountKey(**mapped_params))
        case "Aad":
            auth_model = AADAuthTypeConnectionProperties()

    return auth_model

# --------------------------------------------------------------------------------------------
# Connection utilities
# --------------------------------------------------------------------------------------------


def _get_connection_alternate_target_names():
    return ["target", "api_base", "url", "azure_endpoint", "endpoint"]


def _get_valid_connection_types():
    return [conn_type.value for conn_type in ConnectionCategory]


def _extract_connection_target(conn_dict: Dict[str, Any]) -> str | None:
    connection_target = None
    for target_name in _get_connection_alternate_target_names():
        if target_name in conn_dict:
            connection_target = conn_dict[target_name]
            break
    return connection_target


def _extract_auth_type_and_params(conn_dict: Dict[str, Any]) -> tuple[str, Dict[str, Any]]:
    auth_type = "None"
    auth_params = {}
    if 'credentials' in conn_dict:
        auth_type = conn_dict['credentials'].get('type', None)
        auth_params = {k: v for k, v in conn_dict['credentials'].items() if k != 'type'}
    elif 'api_key' in conn_dict:
        auth_type = 'ApiKey'
        auth_params = {'key': conn_dict['api_key']}
    return (auth_type, auth_params)


def get_connection_category(connection_type: str):
    connection_category = None
    normalized_name = snake_to_camel(connection_type)
    try:
        connection_category = ConnectionCategory(normalized_name)
    except ValueError:
        from . _ml_utils import _ML_CONNECTION_TYPE_TO_COGNITIVE_SERVICES_CONNECTION_TYPE
        connection_category = _ML_CONNECTION_TYPE_TO_COGNITIVE_SERVICES_CONNECTION_TYPE.get(normalized_name, None)
    return connection_category


def compose_connection(
    connection_type: str,
    target: str | None,
    auth_type: str,
    auth_params: Dict[str, Any],
    metadata: Optional[Dict[str, str]] = None
):
    connection_category = get_connection_category(connection_type)
    if connection_category is None:
        raise InvalidArgumentValueError(
            f"Invalid connection type '{connection_type}'. ",
            recommendation=[
                "Verify that the connection type property is set to one of the following values:",
                ', '.join(_get_valid_connection_types())])

    connection = get_auth_model_connection_properties(connection_category, auth_type, auth_params)
    if connection is None:
        raise InvalidArgumentValueError(
            f"Invalid or unsupported auth type '{auth_type}' for connection type '{connection_type}'. ",
            recommendation=[
                "Verify that the auth type property is set to a valid value supported for this connection type."
            ])
    connection.category = connection_category
    connection.target = target
    connection.metadata = metadata or {}

    return connection


def _load_connection_from_dict(conn_dict: Dict[str, Any]):
    connection_type = conn_dict.get("type", None)
    if connection_type is None:
        raise InvalidArgumentValueError(
            "Connection type is required but not found in the provided data.",
            recommendation=[
                "Ensure that the 'type' property is specified in the connection definition."
            ]
        )
    target = _extract_connection_target(conn_dict)
    auth_type, auth_params = _extract_auth_type_and_params(conn_dict)
    return compose_connection(
        connection_type=connection_type,
        target=target,
        auth_type=auth_type,
        auth_params=auth_params,
        metadata=conn_dict.get("metadata", {})
    )


def load_connection_from_source(source: Union[str, PathLike, IO[AnyStr]]):
    """
    Load a connection from a JSON or YAML file or string.
    """
    conn_dict = _load_source_as_dict(source)
    return _load_connection_from_dict(conn_dict)


def _get_as_streamed_context_manager(source: Union[str, PathLike, IO[AnyStr]]):
    from pathlib import Path
    from contextlib import nullcontext
    from io import StringIO
    if isinstance(source, PathLike):
        return open(source, 'r', encoding='utf-8')
    if isinstance(source, str):
        if Path(source).is_file():
            return open(source, 'r', encoding='utf-8')
        return nullcontext(StringIO(source))
    return nullcontext(StringIO(""))


def _load_source_as_dict(source: Union[str, PathLike, IO[AnyStr]]) -> dict:
    import json
    loaded_dict = None
    load_errors = []
    try:
        with _get_as_streamed_context_manager(source) as data_stream:
            loaded_dict = json.load(data_stream)
    except (json.JSONDecodeError) as json_error:
        load_errors.append(str(json_error))
        import yaml
        try:
            with _get_as_streamed_context_manager(source) as data_stream:
                loaded_dict = yaml.safe_load(data_stream)
        except yaml.YAMLError as yaml_error:
            load_errors.append(str(yaml_error))

    if isinstance(loaded_dict, dict):
        return loaded_dict
    if len(load_errors) > 0:
        error_messages = "\n".join(load_errors)
        exception_error_message = f"Errors encountered while loading data source:\n{error_messages}"
    else:
        exception_error_message = "Unknown error encountered while loading data source."
    raise InvalidArgumentValueError(exception_error_message,
                                    recommendation="Check the format of data and/or file.")



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=unused-argument, line-too-long
from azure.cli.core.util import user_confirmation
from ..utils.validators import (
    validate_citus_cluster,
    validate_resource_group,
    validate_virtual_endpoint_name_availability,
)


def virtual_endpoint_create_func(cmd, client, resource_group_name, server_name, virtual_endpoint_name, endpoint_type, members):
    validate_resource_group(resource_group_name)
    validate_citus_cluster(cmd, resource_group_name, server_name)
    validate_virtual_endpoint_name_availability(cmd, virtual_endpoint_name)

    parameters = {
        'properties': {
            'endpointType': endpoint_type,
            'members': [members]
        }
    }

    return client.begin_create(
        resource_group_name,
        server_name,
        virtual_endpoint_name,
        parameters)


def virtual_endpoint_show_func(cmd, client, resource_group_name, server_name, virtual_endpoint_name):
    validate_resource_group(resource_group_name)
    validate_citus_cluster(cmd, resource_group_name, server_name)

    return client.get(
        resource_group_name,
        server_name,
        virtual_endpoint_name)


def virtual_endpoint_list_func(cmd, client, resource_group_name, server_name):
    validate_resource_group(resource_group_name)
    validate_citus_cluster(cmd, resource_group_name, server_name)

    return client.list_by_server(
        resource_group_name,
        server_name)


def virtual_endpoint_delete_func(cmd, client, resource_group_name, server_name, virtual_endpoint_name, yes=False):
    validate_resource_group(resource_group_name)
    validate_citus_cluster(cmd, resource_group_name, server_name)

    if not yes:
        user_confirmation(
            "Are you sure you want to delete the virtual endpoint '{0}' in resource group '{1}'".format(virtual_endpoint_name,
                                                                                                        resource_group_name), yes=yes)

    return client.begin_delete(
        resource_group_name,
        server_name,
        virtual_endpoint_name)


def virtual_endpoint_update_func(cmd, client, resource_group_name, server_name, virtual_endpoint_name, endpoint_type, members):
    validate_resource_group(resource_group_name)
    validate_citus_cluster(cmd, resource_group_name, server_name)

    parameters = {
        'properties': {
            'endpointType': endpoint_type,
            'members': [members]
        }
    }

    return client.begin_update(
        resource_group_name,
        server_name,
        virtual_endpoint_name,
        parameters)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=protected-access

from azure.cli.core.aaz import AAZStrArg, AAZListArg
from azure.cli.command_modules.monitor.actions import AAZCustomListArg
from azure.cli.command_modules.monitor.aaz.latest.monitor.action_group.test_notifications._create \
    import Create as _ActionGroupTestNotificationCreate
from azure.cli.command_modules.monitor.operations.action_groups import update_action_group_receivers


class ActionGroupTestNotificationCreate(_ActionGroupTestNotificationCreate):

    def __init__(self, *args, **kwargs):
        from azure.cli.command_modules.monitor.transformers import action_group_list_table
        super().__init__(*args, **kwargs)
        self.table_transformer = action_group_list_table

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.arm_role_receivers._registered = False
        args_schema.automation_runbook_receivers._registered = False
        args_schema.azure_app_push_receivers._registered = False
        args_schema.azure_function_receivers._registered = False
        args_schema.email_receivers._registered = False

        args_schema.event_hub_receivers._registered = False
        args_schema.itsm_receivers._registered = False
        args_schema.logic_app_receivers._registered = False
        args_schema.sms_receivers._registered = False
        args_schema.voice_receivers._registered = False
        args_schema.webhook_receivers._registered = False
        args_schema.receiver_actions = AAZCustomListArg(
            options=["--add-actions"],
            singular_options=["--add-action", "-a"],
            help='''
        Add receivers to the action group.\n\n
        Usage:   --add-action TYPE NAME [ARG ...]\n\n
        Email:\n\n
            Format:     --add-action email NAME EMAIL_ADDRESS [usecommonalertschema]\n\n
            Example:    --add-action email bob bob@contoso.com\n\n
        SMS:\n\n
            Format:     --add-action sms NAME COUNTRY_CODE PHONE_NUMBER\n\n
            Example:    --add-action sms charli 1 5551234567\n\n
        Webhook:\n\n
            Format:     --add-action webhook NAME URI [useaadauth OBJECT_ID IDENTIFIER URI] [usecommonalertschema]\n\n
            Example:    --add-action https://www.contoso.com/alert useaadauth testobj http://identifier usecommonalertschema\n\n
        Arm Role:\n\n
            Format:     --add-action armrole NAME ROLE_ID [usecommonalertschema]\n\n
            Example:    --add-action armole owner_role 8e3af657-a8ff-443c-a75c-2fe8c4bcb635\n\n
        Azure App Push:\n\n
            Format:     --add-action azureapppush NAME EMAIL_ADDRESS\n\n
            Example:    --add-action azureapppush test_apppush bob@contoso.com\n\n
        ITSM:\n\n
            Format:     --add-action itsm NAME WORKSPACE_ID CONNECTION_ID TICKET_CONFIGURATION REGION\n\n
            Example:    --add-action itsm test_itsm test_workspace test_conn ticket_blob useast\n\n
        Automation runbook:\n\n
            Format:     --add-action automationrunbook NAME AUTOMATION_ACCOUNT_ID RUNBOOK_NAME WEBHOOK_RESOURCE_ID SERVICE_URI [isglobalrunbook] [usecommonalertschema]\n\n
            Example:    --add-action automationrunbook test_runbook test_acc test_book test_webhook test_rsrc http://example.com isglobalrunbook usecommonalertschema\n\n
        Voice:\n\n
            Format:     --add-action voice NAME COUNTRY_CODE PHONE_NUMBER\n\n
            Example:    --add-action voice charli 1 4441234567\n\n
        Logic App:\n\n
            Format:     --add-action logicapp NAME RESOURCE_ID CALLBACK_URL [usecommonalertschema]\n\n
            Example:    --add-action logicapp test_logicapp test_rsrc http://callback\n\n
        Azure Function:\n\n
            Format:     --add-action azurefunction NAME FUNCTION_APP_RESOURCE_ID FUNCTION_NAME HTTP_TRIGGER_URL [usecommonalertschema]\n\n
            Example:    --add-action azurefunction test_function test_rsrc test_func http://trigger usecommonalertschema\n\n
        Event Hub:\n\n
            Format:     --action eventhub NAME SUBSCRIPTION_ID EVENT_HUB_NAME_SPACE EVENT_HUB_NAME [usecommonalertschema]\n\n
            Example:    --action eventhub test_eventhub 5def922a-3ed4-49c1-b9fd-05ec533819a3 eventhubNameSpace testEventHubName usecommonalertschema\n\n
        Multiple actions can be specified by using more than one `--add-action` argument.\n\n
        'useaadauth', 'isglobalrunbook' and 'usecommonalertschema' are optional arguements that only need to be passed to set the respective parameter to True.\n\n
        If the 'useaadauth' argument is passed, then the OBJECT_ID and IDENTIFIER_URI values are required as well.''',
        )
        args_schema.receiver_actions.Element = AAZListArg()
        args_schema.receiver_actions.Element.Element = AAZStrArg()
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        update_action_group_receivers(args)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import json


DOUBLE_QUOTE = '\"'
BACKSLASH = '\\'
DOUBLE_SLASH = '//'
MULTILINE_COMMENT_START = '/*'
MULTILINE_COMMENT_END = '*/'
NEW_LINE = '\n'


def parse_json_with_comments(json_string):
    try:
        return json.loads(json_string)

    except json.JSONDecodeError:
        return json.loads(__strip_json_comments(json_string))


def __strip_json_comments(json_string):
    current_index = 0
    length = len(json_string)
    result = []

    while current_index < length:
        # Single line comment
        if json_string[current_index:current_index + 2] == DOUBLE_SLASH:
            current_index = __find_next_newline_index(json_string, current_index, length)
            if current_index < length and json_string[current_index] == NEW_LINE:
                result.append(NEW_LINE)

        # Multi-line comment
        elif json_string[current_index:current_index + 2] == MULTILINE_COMMENT_START:
            current_index = __find_next_multiline_comment_end(json_string, current_index + 2, length)

        # String literal
        elif json_string[current_index] == DOUBLE_QUOTE:
            literal_start_index = current_index
            current_index = __find_next_double_quote_index(json_string, current_index + 1, length)

            result.extend(json_string[literal_start_index:current_index + 1])
        else:
            result.append(json_string[current_index])

        current_index += 1

    return "".join(result)


def __is_escaped(json_string, char_index):
    backslash_count = 0
    index = char_index - 1
    while index >= 0 and json_string[index] == BACKSLASH:
        backslash_count += 1
        index -= 1

    return backslash_count % 2 == 1


def __find_next_newline_index(json_string, start_index, end_index):
    index = start_index

    while index < end_index:
        if json_string[index] == NEW_LINE:
            return index

        index += 1

    return index


def __find_next_double_quote_index(json_string, start_index, end_index):
    index = start_index
    while index < end_index:
        if json_string[index] == DOUBLE_QUOTE and not __is_escaped(json_string, index):
            return index

        index += 1

    raise ValueError("Unterminated string literal")


def __find_next_multiline_comment_end(json_string, start_index, end_index):
    index = start_index
    while index < end_index - 1:
        if json_string[index:index + 2] == MULTILINE_COMMENT_END:
            return index + 1

        index += 1

    raise ValueError("Unterminated multi-line comment")



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

import os
import logging


def save_payload(config_dir, payload):
    """
    Save a telemetry payload to the telemetry cache directory under the given configuration directory
    """
    logger = logging.getLogger('telemetry.save')

    if payload:
        cache_saver, cache_dir = _create_rotate_file_logger(config_dir)
        if cache_saver:
            cache_saver.info(payload)
            logger.info('Save telemetry record of length %d in cache file under %s', len(payload), cache_dir)

            return cache_dir
    return None


def _create_rotate_file_logger(log_dir):
    from logging.handlers import RotatingFileHandler
    from datetime import datetime
    now = datetime.now()
    cache_name = os.path.join(log_dir, 'telemetry', now.strftime('%Y%m%d%H%M%S%f')[:-3], 'cache')
    try:
        if not os.path.exists(os.path.dirname(cache_name)):
            os.makedirs(os.path.dirname(cache_name))

        handler = RotatingFileHandler(cache_name, maxBytes=128 * 1024, backupCount=100)
        handler.setLevel(logging.INFO)
        handler.setFormatter(logging.Formatter(fmt='%(asctime)s,%(message)s',
                                               datefmt='%Y-%m-%dT%H:%M:%S'))

        logger = logging.Logger(name='telemetry_cache', level=logging.INFO)
        logger.addHandler(handler)
        return logger, os.path.dirname(cache_name)
    except OSError as err:
        logging.getLogger('telemetry.save').warning('Fail to create telemetry cache directory for %s. Reason %s.',
                                                    cache_name,
                                                    err)
        return None



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# AZURE CLI EventHub - NAMESPACE TEST DEFINITIONS

import time

from azure.cli.testsdk import (ScenarioTest, ResourceGroupPreparer, KeyVaultPreparer)


# pylint: disable=line-too-long
# pylint: disable=too-many-lines


class EHNamespaceEntityCURDScenarioTest(ScenarioTest):
    from azure.cli.testsdk.scenario_tests import AllowLargeResponse

    @AllowLargeResponse()
    @ResourceGroupPreparer(name_prefix='cli_test_eh_namespace')
    def test_eh_create_update(self, resource_group):
        self.kwargs.update({
            'loc': 'westus2',
            'rg': resource_group,
            'namespacename': self.create_random_name(prefix='eventhubs-nscli', length=20),
            'namespacename1': self.create_random_name(prefix='eventhubs-nscli', length=20),
            'namespacename2': self.create_random_name(prefix='eventhubs-nscli', length=20),
            'tags': {'tag1: value1', 'tag2: value2'},
            'sku': 'Standard',
            'tier': 'Standard',
            'eventhubname1': self.create_random_name(prefix='eventhubs-eventhubcli', length=25),
            'eventhubname2': self.create_random_name(prefix='eventhubs-eventhubcli', length=25),
            'eventhubname3': self.create_random_name(prefix='eventhubs-eventhubcli', length=25),
            'eventhubname4': self.create_random_name(prefix='eventhubs-eventhubcli', length=25),
            'eventhubname5': self.create_random_name(prefix='eventhubs-eventhubcli', length=25),
            'eventhubname6': self.create_random_name(prefix='eventhubs-eventhubcli', length=25),
            'isautoinflateenabled': 'True',
            'maximumthroughputunits': 4,
            'messageretentionindays': 4,
            'partitioncount': 4,
            'identity1': self.create_random_name(prefix='eh-identity1', length=20),
            'identity2': self.create_random_name(prefix='eh-identity2', length=20),
            'destinationname': 'EventHubArchive.AzureBlockBlob',
            'storageaccount': self.create_random_name(prefix='storageehnscli', length=20),
            'containername': self.create_random_name(prefix='container-nscli', length=20),
            'blobcontainer': 'container01',
            'storageaccount1': self.create_random_name(prefix='storageehnscli', length=20),
            'containername1': self.create_random_name(prefix='container-nscli', length=20),
            'blobcontainer1': 'container02',
            'storageaccount2': self.create_random_name(prefix='storageehnscli', length=20),
            'containername2': self.create_random_name(prefix='container-nscli', length=20),
            'blobcontainer2': 'container03',
            'capturesizelimit': 314572799,
            'archinvenameformat': '{Namespace}/{EventHub}/{PartitionId}/{Year}/{Month}/{Day}/{Hour}/{Minute}/{Second}'
        })
        storage_account = self.cmd('storage account create -n {storageaccount} -g {rg} -l westus --sku Standard_LRS').get_output_in_json()

        self.kwargs.update({'storageid': storage_account['id']})

        container = self.cmd('storage container create -n {containername} -g {rg} --account-name {storageaccount}').get_output_in_json()

        self.cmd('eventhubs namespace create --resource-group {rg} --name {namespacename} --location {loc} --tags {tags} --sku {sku} --enable-auto-inflate {isautoinflateenabled} --maximum-throughput-units {maximumthroughputunits}')

        eh1 = self.cmd('eventhubs eventhub create -g {rg} -n {eventhubname1} --namespace-name {namespacename} --retention-time-in-hours 24 --partition-count 15 --enable-capture true --capture-interval 100 --capture-size-limit 314572799 '
                       '--destination-name {destinationname} --storage-account {storageid} --blob-container {containername} --archive-name-format {archinvenameformat} --cleanup-policy Delete').get_output_in_json()

        self.assertEqual(eh1['name'], self.kwargs['eventhubname1'])
        self.assertEqual(eh1['retentionDescription']['retentionTimeInHours'], 24)
        self.assertEqual(eh1['partitionCount'], 15)
        self.assertEqual(eh1['captureDescription']['enabled'], True)
        self.assertEqual(eh1['captureDescription']['intervalInSeconds'], 100)
        self.assertEqual(eh1['captureDescription']['sizeLimitInBytes'], self.kwargs['capturesizelimit'])
        self.assertEqual(eh1['captureDescription']['destination']['archiveNameFormat'], self.kwargs['archinvenameformat'])
        self.assertEqual(eh1['captureDescription']['destination']['blobContainer'], self.kwargs['containername'])
        self.assertEqual(eh1['captureDescription']['destination']['name'], self.kwargs['destinationname'])
        self.assertEqual(eh1['captureDescription']['destination']['storageAccountResourceId'], self.kwargs['storageid'])

        eh1 = self.cmd(
            'eventhubs eventhub update -g {rg} -n {eventhubname1} --namespace-name {namespacename} --enable-capture false').get_output_in_json()

        self.assertEqual(eh1['name'], self.kwargs['eventhubname1'])
        self.assertEqual(eh1['messageRetentionInDays'], 1)
        self.assertEqual(eh1['partitionCount'], 15)
        self.assertEqual(eh1['captureDescription']['enabled'], False)
        self.assertEqual(eh1['captureDescription']['intervalInSeconds'], 100)
        self.assertEqual(eh1['captureDescription']['sizeLimitInBytes'], self.kwargs['capturesizelimit'])
        self.assertEqual(eh1['captureDescription']['destination']['archiveNameFormat'],
                         self.kwargs['archinvenameformat'])
        self.assertEqual(eh1['captureDescription']['destination']['blobContainer'], self.kwargs['containername'])
        self.assertEqual(eh1['captureDescription']['destination']['name'], self.kwargs['destinationname'])
        self.assertEqual(eh1['captureDescription']['destination']['storageAccountResourceId'], self.kwargs['storageid'])

        eh2 = self.cmd(
            'eventhubs eventhub create -g {rg} -n {eventhubname2} --namespace-name {namespacename} --partition-count 15').get_output_in_json()

        self.assertEqual(eh2['name'], self.kwargs['eventhubname2'])
        self.assertEqual(eh2['partitionCount'], 15)

        eh2 = self.cmd(
            'eventhubs eventhub update -g {rg} -n {eventhubname2} --namespace-name {namespacename} --enable-capture --capture-interval 100 --capture-size-limit 314572799 '
            '--destination-name {destinationname} --storage-account {storageid} --blob-container {containername} --archive-name-format {archinvenameformat}').get_output_in_json()

        self.assertEqual(eh2['name'], self.kwargs['eventhubname2'])
        self.assertEqual(eh2['partitionCount'], 15)
        self.assertEqual(eh2['captureDescription']['enabled'], True)
        self.assertEqual(eh2['captureDescription']['intervalInSeconds'], 100)
        self.assertEqual(eh2['captureDescription']['sizeLimitInBytes'], self.kwargs['capturesizelimit'])
        self.assertEqual(eh2['captureDescription']['destination']['archiveNameFormat'],
                         self.kwargs['archinvenameformat'])
        self.assertEqual(eh2['captureDescription']['destination']['blobContainer'], self.kwargs['containername'])
        self.assertEqual(eh2['captureDescription']['destination']['name'], self.kwargs['destinationname'])
        self.assertEqual(eh2['captureDescription']['destination']['storageAccountResourceId'], self.kwargs['storageid'])

        self.cmd(
            'eventhubs namespace create --resource-group {rg} --name {namespacename1} --location {loc} --tags {tags} --sku Premium ')
        eh3 = self.cmd(
            'eventhubs eventhub create -g {rg} -n {eventhubname3} --namespace-name {namespacename1} --cleanup-policy Delete --retention-time 7 ').get_output_in_json()

        self.assertEqual(eh3['name'], self.kwargs['eventhubname3'])
        self.assertEqual(eh3['retentionDescription']['cleanupPolicy'], "Delete")
        self.assertEqual(eh3['retentionDescription']['retentionTimeInHours'], 7)

        eh4 = self.cmd(
            'eventhubs eventhub create -g {rg} -n {eventhubname4} --namespace-name {namespacename1} --cleanup-policy Compact').get_output_in_json()
        self.assertEqual(eh4['name'], self.kwargs['eventhubname4'])
        self.assertEqual(eh4['retentionDescription']['cleanupPolicy'], "Compact")

        storage_account1 = self.cmd(
            'storage account create -n {storageaccount1} -g {rg} -l westus --sku Standard_LRS').get_output_in_json()

        self.kwargs.update({'storageid1': storage_account1['id']})

        container = self.cmd(
            'storage container create -n {containername1} -g {rg} --account-name {storageaccount1}').get_output_in_json()
        eh5 = self.cmd(
            'eventhubs eventhub create -g {rg} -n {eventhubname5} --namespace-name {namespacename} --partition-count 15 --enable-capture true --capture-interval 100 --capture-size-limit 314572799 '
            '--destination-name {destinationname} --storage-account {storageid1} --blob-container {containername1} --archive-name-format {archinvenameformat} --cleanup-policy Compact').get_output_in_json()

        self.assertEqual(eh5['name'], self.kwargs['eventhubname5'])
        self.assertEqual(eh5['partitionCount'], 15)
        self.assertEqual(eh5['captureDescription']['enabled'], True)
        self.assertEqual(eh5['captureDescription']['intervalInSeconds'], 100)
        self.assertEqual(eh5['captureDescription']['sizeLimitInBytes'], self.kwargs['capturesizelimit'])
        self.assertEqual(eh5['captureDescription']['destination']['archiveNameFormat'],
                         self.kwargs['archinvenameformat'])
        self.assertEqual(eh5['captureDescription']['destination']['blobContainer'], self.kwargs['containername1'])
        self.assertEqual(eh5['captureDescription']['destination']['name'], self.kwargs['destinationname'])
        self.assertEqual(eh5['captureDescription']['destination']['storageAccountResourceId'], self.kwargs['storageid1'])

        identity2 = self.cmd('identity create --name {identity2} --resource-group {rg}').get_output_in_json()
        self.kwargs.update({'id2': identity2['id']})
        self.kwargs.update({'id3': identity2['principalId']})

        storage_account2 = self.cmd(
            'storage account create -n {storageaccount2} -g {rg} -l westus --sku Standard_RAGRS ').get_output_in_json()

        self.kwargs.update({'storageid2': storage_account2['id']})

        container = self.cmd(
            'storage container create -n {containername2} -g {rg} --account-name {storageaccount2}').get_output_in_json()

        '''self.cmd(
            'az role assignment create --assignee {id3} --role "Storage Blob Data Contributor" --scope {storageid2}')'''

        self.cmd(
            'eventhubs namespace create --resource-group {rg} --name {namespacename2} --location {loc} --tags {tags} --sku Premium '
            '--mi-user-assigned {id2}').get_output_in_json()
        time.sleep(200)

        eh6 = self.cmd(
            'eventhubs eventhub create -g {rg} -n {eventhubname6} --namespace-name {namespacename2} --partition-count 15 --enable-capture true --capture-interval 100 --capture-size-limit 314572799 '
            '--destination-name {destinationname} --storage-account {storageid2} --blob-container {containername2} --archive-name-format {archinvenameformat} '
            '--mi-user-assigned {id2}').get_output_in_json()

        self.cmd('eventhubs eventhub delete --resource-group {rg} --namespace-name {namespacename1} --name {eventhubname4}')
        self.cmd('eventhubs namespace delete --resource-group {rg} --name {namespacename}')
        self.cmd('eventhubs namespace delete --resource-group {rg} --name {namespacename1}')
        self.cmd('eventhubs namespace delete --resource-group {rg} --name {namespacename2}')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.testsdk import ScenarioTest, ResourceGroupPreparer, StorageAccountPreparer, KeyVaultPreparer


class AmsEncryptionTests(ScenarioTest):
    @ResourceGroupPreparer()
    @StorageAccountPreparer(parameter_name='storage_account_for_create')
    @KeyVaultPreparer(location='centralus', additional_params='--enable-purge-protection')
    def test_ams_encryption_set_show(self, resource_group, storage_account_for_create, key_vault):
        amsname = self.create_random_name(prefix='ams', length=12)

        self.kwargs.update({
            'amsname': amsname,
            'storageAccount': storage_account_for_create,
            'location': 'centralus',
            'keySource': 'CustomerKey',
            'keyVault': key_vault,
            'keyName': self.create_random_name(prefix='ams', length=12),
            'identityPermissions': "get unwrapkey wrapkey",
        })

        account_result = self.cmd('az ams account create -n {amsname} -g {rg} --storage-account {storageAccount} -l {location} --mi-system-assigned --default-action Allow', checks=[
            self.check('name', '{amsname}'),
            self.check('location', 'Central US')
        ])

        key_vault_result = self.cmd('keyvault key create --name {keyName} --vault-name {keyVault}')

        self.kwargs['keyVaultId'] = key_vault_result.get_output_in_json()['key']['kid']
        self.kwargs['principalId'] = account_result.get_output_in_json()['identity']['principalId']

        self.cmd('keyvault set-policy --object-id {principalId} --name {keyVault} --key-permissions {identityPermissions}')

        self.cmd('az ams account encryption set -a {amsname} -g {rg} --key-type {keySource} --key-identifier {keyVaultId}', checks=[
            self.check('name', '{amsname}'),
        ])

        self.cmd('az ams account encryption show -a {amsname} -g {rg}', checks=[
            self.check('keyVaultProperties.keyIdentifier', '{keyVaultId}')
        ])

        self.cmd('az ams account delete -n {amsname} -g {rg}')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from .exceptions import AAZUnknownFieldError, AAZUndefinedValueError
from knack.help import _print_indent, REQUIRED_TAG, FIRST_LINE_PREFIX
from knack.util import status_tag_messages

try:
    from knack.preview import _PREVIEW_TAG as PREVIEW_TAG
except ImportError:
    PREVIEW_TAG = '[Preview]'

try:
    from knack.experimental import _EXPERIMENTAL_TAG as EXPERIMENTAL_TAG
except ImportError:
    EXPERIMENTAL_TAG = '[Experimental]'

# pylint: disable=protected-access

shorthand_help_messages = {
    "show-help": 'Try "??" to show more.',
    "short-summary": 'Support shorthand-syntax, json-file and yaml-file.',
    "short-summary-anytype": 'Support shorthand-syntax(full value only), json-file and yaml-file.',
    "long-summary": 'See https://aka.ms/cli-shorthand for more about shorthand syntax.'
}


class AAZShowHelp(BaseException):

    def __init__(self, *keys):
        super().__init__()
        self.keys = keys
        self.schema = None

    def show(self):
        from ._arg import AAZObjectArg, AAZDictArg, AAZFreeFormDictArg, AAZListArg, AAZBaseArg
        assert self.schema is not None and isinstance(self.schema, AAZBaseArg)
        schema = self.schema
        schema_key = self.keys[0]
        idx = 1
        while idx < len(self.keys):
            key = self.keys[idx]
            if isinstance(schema, AAZObjectArg):
                try:
                    schema = schema[key]  # pylint: disable=unsubscriptable-object
                except AAZUndefinedValueError:
                    # show the help of current schema
                    break
                key = f'.{key}'
            elif isinstance(schema, (AAZDictArg, AAZFreeFormDictArg)):
                try:
                    schema = schema.Element
                except AAZUnknownFieldError:
                    # show the help of current schema
                    break
                key = '{}'
            elif isinstance(schema, AAZListArg):
                try:
                    schema = schema.Element
                except AAZUnknownFieldError:
                    # show the help of current schema
                    break
                key = '[]'
            else:
                # show the help of current schema
                break
            schema_key += key
            idx += 1

        self._print_schema_basic(schema_key, schema)

        if isinstance(schema, AAZObjectArg):
            self._print_object_props_schema(schema, title="Object Properties")
        elif isinstance(schema, AAZDictArg) and isinstance(schema.Element, AAZObjectArg):
            self._print_object_props_schema(schema.Element, title="Dict Element Properties")
        elif isinstance(schema, AAZListArg) and isinstance(schema.Element, AAZObjectArg):
            self._print_object_props_schema(schema.Element, title="List Element Properties")

        _print_indent("")

    @classmethod
    def _print_schema_basic(cls, key, schema):
        indent = 0
        _print_indent('\nArgument', indent)

        help_tags = cls._build_schema_tags(schema, preview=False, experimental=False)
        schema_type = schema._type_in_help
        if help_tags:
            _print_indent(f"{key} {help_tags}{FIRST_LINE_PREFIX}{schema_type}", indent=1)
        else:
            _print_indent(f"{key}{FIRST_LINE_PREFIX}{schema_type}", indent=1)

        short_summary = cls._build_short_summary(schema)
        if short_summary:
            _print_indent(short_summary, indent=2)

        long_summary = cls._build_long_summary(schema)
        if long_summary:
            _print_indent(long_summary, indent=2)

    @classmethod
    def _print_object_props_schema(cls, schema, title):
        LINE_FORMAT = '{name}{padding}{tags}{separator}{short_summary}'

        _print_indent(f"\n{title}")
        layouts = []
        max_header_len = 0

        for prop_schema in schema._fields.values():
            if not prop_schema._registered:
                # ignore unregistered args
                continue

            prop_tags = cls._build_schema_tags(prop_schema)
            prop_name = ' '.join(prop_schema._options)

            prop_short_summary = cls._build_short_summary(prop_schema, is_prop=True)

            prop_group_name = prop_schema._arg_group or ""
            header_len = len(prop_name) + len(prop_tags) + (1 if prop_tags else 0)
            max_header_len = max(max_header_len, header_len)
            layouts.append({
                "name": prop_name,
                "tags": prop_tags,
                'separator': FIRST_LINE_PREFIX if prop_short_summary else '',
                'short_summary': prop_short_summary,
                'group_name': prop_group_name,
                'header_len': header_len
            })

        layouts.sort(key=lambda a: (a['group_name'], 0 if a['tags'] else 1, a['tags'], a['name']))
        for layout in layouts:
            if layout['tags']:
                pad_len = max_header_len - layout['header_len'] + 1
            else:
                pad_len = max_header_len - layout['header_len']
            layout['padding'] = ' ' * pad_len
            _print_indent(
                LINE_FORMAT.format(**layout),
                indent=1,
                subsequent_spaces=max_header_len + 4 + len(FIRST_LINE_PREFIX) - 1
            )

    @staticmethod
    def _build_schema_tags(schema, required=True, preview=True, experimental=False):
        preview = PREVIEW_TAG if preview and schema._is_preview else ''
        experimental = EXPERIMENTAL_TAG if experimental and schema._is_experimental else ''
        required = REQUIRED_TAG if required and schema._required else ''
        tags = ' '.join([x for x in [required, experimental, preview] if x])
        return tags

    @staticmethod
    def _build_short_summary(schema, is_prop=False):
        from ._arg import AAZSimpleTypeArg, AAZCompoundTypeArg
        short_summary = schema._help.get("short-summary", "")

        if isinstance(schema, AAZSimpleTypeArg) and schema.enum:
            choices = schema.enum.to_choices()
            if choices:
                if short_summary:
                    short_summary += '  '
                short_summary += 'Allowed values: {}.'.format(', '.join(sorted([str(x) for x in choices])))
        elif isinstance(schema, AAZCompoundTypeArg):
            if is_prop:
                if short_summary:
                    short_summary += '  '
                short_summary += shorthand_help_messages['show-help']
        return short_summary

    @staticmethod
    def _build_long_summary(schema):
        from ._arg import AAZCompoundTypeArg
        lines = []
        long_summary = schema._help.get("long-summary", "")
        if long_summary:
            lines.append(long_summary)

        if isinstance(schema, AAZCompoundTypeArg):
            lines.append(shorthand_help_messages['long-summary'])

        if schema._is_preview:
            preview = status_tag_messages['preview'].format("This argument")
            lines.append(preview)
        if schema._is_experimental:
            experimental = status_tag_messages['experimental'].format("This argument")
            lines.append(experimental)
        return '\n'.join(lines)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from azure.cli.core.commands.client_factory import get_mgmt_service_client
from azure.cli.core.profiles import ResourceType
from azure.cli.core.auth.identity import get_environment_credential, AZURE_CLIENT_ID

# pylint: disable=import-outside-toplevel

RM_URI_OVERRIDE = 'AZURE_CLI_RDBMS_RM_URI'
SUB_ID_OVERRIDE = 'AZURE_CLI_RDBMS_SUB_ID'


def get_mysql_flexible_management_client(cli_ctx, **_):
    from os import getenv
    from azure.mgmt.mysqlflexibleservers import MySQLManagementClient

    # Allow overriding resource manager URI using environment variable
    # for testing purposes. Subscription id is also determined by environment
    # variable.
    rm_uri_override = getenv(RM_URI_OVERRIDE)
    if rm_uri_override:
        client_id = getenv(AZURE_CLIENT_ID)
        if client_id:
            credentials = get_environment_credential()
        else:
            from msrest.authentication import Authentication  # pylint: disable=import-error
            credentials = Authentication()

        return MySQLManagementClient(
            subscription_id=getenv(SUB_ID_OVERRIDE),
            base_url=rm_uri_override,
            credential=credentials)
    # Normal production scenario.
    return get_mgmt_service_client(cli_ctx, MySQLManagementClient)


def get_mysql_flexible_management_client_by_sub(cli_ctx, subscription_id, **_):
    from os import getenv
    from azure.mgmt.mysqlflexibleservers import MySQLManagementClient

    # Allow overriding resource manager URI using environment variable
    rm_uri_override = getenv(RM_URI_OVERRIDE)
    if rm_uri_override:
        client_id = getenv(AZURE_CLIENT_ID)
        if client_id:
            credentials = get_environment_credential()
        else:
            from msrest.authentication import Authentication  # pylint: disable=import-error
            credentials = Authentication()

        return MySQLManagementClient(
            subscription_id=subscription_id,
            base_url=rm_uri_override,
            credential=credentials)
    # Normal production scenario.
    return get_mgmt_service_client(cli_ctx, MySQLManagementClient, subscription_id=subscription_id)


def get_mysql_management_client(cli_ctx, **_):
    from os import getenv
    from azure.mgmt.rdbms.mysql import MySQLManagementClient

    # Allow overriding resource manager URI using environment variable
    # for testing purposes. Subscription id is also determined by environment
    # variable.
    rm_uri_override = getenv(RM_URI_OVERRIDE)
    if rm_uri_override:
        client_id = getenv(AZURE_CLIENT_ID)
        if client_id:
            credentials = get_environment_credential()
        else:
            from msrest.authentication import Authentication  # pylint: disable=import-error
            credentials = Authentication()

        return MySQLManagementClient(
            subscription_id=getenv(SUB_ID_OVERRIDE),
            base_url=rm_uri_override,
            credential=credentials)
    # Normal production scenario.
    return get_mgmt_service_client(cli_ctx, MySQLManagementClient)


# Meru operations for flexible servers
def cf_mysql_flexible_servers(cli_ctx, _):
    return get_mysql_flexible_management_client(cli_ctx).servers


def cf_mysql_flexible_firewall_rules(cli_ctx, _):
    return get_mysql_flexible_management_client(cli_ctx).firewall_rules


def cf_mysql_flexible_config(cli_ctx, _):
    return get_mysql_flexible_management_client(cli_ctx).configurations


def cf_mysql_flexible_db(cli_ctx, _):
    return get_mysql_flexible_management_client(cli_ctx).databases


def cf_mysql_flexible_replica(cli_ctx, _):
    return get_mysql_flexible_management_client(cli_ctx).replicas


def cf_mysql_flexible_location_capabilities(cli_ctx, _):
    return get_mysql_flexible_management_client(cli_ctx).location_based_capabilities


def cf_mysql_flexible_log(cli_ctx, _):
    return get_mysql_flexible_management_client(cli_ctx).log_files


def cf_mysql_flexible_backup(cli_ctx, _):
    return get_mysql_flexible_management_client(cli_ctx).long_running_backup


def cf_mysql_flexible_backups(cli_ctx, _):
    return get_mysql_flexible_management_client(cli_ctx).long_running_backups


def cf_mysql_flexible_export(cli_ctx, _):
    return get_mysql_flexible_management_client(cli_ctx).backup_and_export


def cf_mysql_flexible_adadmin(cli_ctx, _):
    return get_mysql_flexible_management_client(cli_ctx).azure_ad_administrators


def cf_mysql_advanced_threat_protection(cli_ctx, _):
    return get_mysql_flexible_management_client(cli_ctx).advanced_threat_protection_settings


def cf_mysql_flexible_maintenances(cli_ctx, _):
    return get_mysql_flexible_management_client(cli_ctx).maintenances


def cf_mysql_check_resource_availability(cli_ctx, _):
    return get_mysql_flexible_management_client(cli_ctx).check_name_availability


def cf_mysql_check_resource_availability_without_location(cli_ctx, _):
    return get_mysql_flexible_management_client(cli_ctx).check_name_availability_without_location


def cf_mysql_flexible_private_dns_zone_suffix_operations(cli_ctx, _):
    return get_mysql_flexible_management_client(cli_ctx).get_private_dns_zone_suffix


def resource_client_factory(cli_ctx, subscription_id=None):
    return get_mgmt_service_client(cli_ctx, ResourceType.MGMT_RESOURCE_RESOURCES, subscription_id=subscription_id)


def private_dns_client_factory(cli_ctx, subscription_id=None):
    from azure.mgmt.privatedns import PrivateDnsManagementClient
    return get_mgmt_service_client(cli_ctx, PrivateDnsManagementClient, subscription_id=subscription_id).private_zones


def private_dns_link_client_factory(cli_ctx, subscription_id=None):
    from azure.mgmt.privatedns import PrivateDnsManagementClient
    return get_mgmt_service_client(cli_ctx, PrivateDnsManagementClient,
                                   subscription_id=subscription_id).virtual_network_links


# Operations for single server
def cf_mysql_servers(cli_ctx, _):
    return get_mysql_management_client(cli_ctx).servers


def cf_mysql_firewall_rules(cli_ctx, _):
    return get_mysql_management_client(cli_ctx).firewall_rules



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=line-too-long

import json
import os
import time
import unittest

from azure.cli.command_modules.appconfig._credential import AppConfigurationCliCredential
from azure.cli.command_modules.appconfig._utils import get_appconfig_data_client
from azure.cli.core._profile import Profile
from azure.cli.core.auth.credential_adaptor import CredentialAdaptor
from azure.cli.core.cloud import get_active_cloud
from azure.cli.core.mock import DummyCli
from knack.util import CLIError
from unittest import mock
from azure.cli.testsdk import (ResourceGroupPreparer, live_only, ScenarioTest)
from azure.cli.testsdk.scenario_tests import AllowLargeResponse
from azure.cli.command_modules.appconfig.tests.latest._test_utils import create_config_store, CredentialResponseSanitizer, get_resource_name_prefix
from azure.cli.command_modules.appconfig._constants import FeatureFlagConstants, KeyVaultConstants

TEST_DIR = os.path.abspath(os.path.join(os.path.abspath(__file__), '..'))
APPCONFIG_AUTH_TOKEN_AUDIENCE = "https://appconfig.azure.com"

TEST_MI_SUBSCRIPTION = {
    "id": "00000000-0000-0000-0000-000000000000",
    "name": "MSI subscription",
    "isDefault": True,
    "state": "Enabled",
    "user": {
        "name": "systemAssignedIdentity"
    },
    "tenantId": "00000000-0000-0000-0000-000000000000",
    "environmentName": "AzureCloud",
    "homeTenantId": "00000000-0000-0000-0000-000000000000",
    "managedByTenants": []
}

TEST_USER_SUBSCRIPTION = {
    "id": "00000000-0000-0000-0000-000000000000",
    "name": "Service Principal Subscription",
    "state": "Enabled",
    "isDefault": True,
     "user": {
    "name": "tst_user@microsoft.com",
    "type": "user"
  },
    "tenantId": "00000000-0000-0000-0000-000000000000",
    "environmentName": "AzureCloud",
    "homeTenantId": "00000000-0000-0000-0000-000000000000",
    "managedByTenants": []
}

def mock_get_active_cloud(cli_ctx=None):
    cloud = get_active_cloud(cli_ctx)
    cloud.endpoints.appconfig_auth_token_audience = APPCONFIG_AUTH_TOKEN_AUDIENCE
    return cloud

class AppConfigAadAuthLiveScenarioTest(ScenarioTest):

    def __init__(self, *args, **kwargs):
        kwargs["recording_processors"] = kwargs.get("recording_processors", []) + [CredentialResponseSanitizer()]
        super().__init__(*args, **kwargs)
    
    @live_only()
    @AllowLargeResponse()
    @ResourceGroupPreparer(parameter_name_for_location='location')
    def test_azconfig_aad_auth(self, resource_group, location):
        aad_store_prefix = get_resource_name_prefix('AADStore')
        config_store_name = self.create_random_name(prefix=aad_store_prefix, length=24)

        location = 'eastus'
        sku = 'standard'
        self.kwargs.update({
            'config_store_name': config_store_name,
            'rg_loc': location,
            'rg': resource_group,
            'sku': sku
        })
        create_config_store(self, self.kwargs)

        # Get connection string and add a key-value and feature flag using the default "key" auth mode
        credential_list = self.cmd(
            'appconfig credential list -n {config_store_name} -g {rg}').get_output_in_json()
        self.kwargs.update({
            'connection_string': credential_list[0]['connectionString']
        })

        # Add a key-value
        entry_key = "Color"
        entry_value = "Red"
        self.kwargs.update({
            'key': entry_key,
            'value': entry_value
        })
        self.cmd('appconfig kv set --connection-string {connection_string} --key {key} --value {value} -y',
                 checks=[self.check('key', entry_key),
                         self.check('value', entry_value)])

        # add a feature flag
        entry_feature = 'Beta'
        internal_feature_key = FeatureFlagConstants.FEATURE_FLAG_PREFIX + entry_feature
        default_description = ""
        default_conditions = "{{\'client_filters\': []}}"
        default_locked = False
        default_state = "off"
        self.kwargs.update({
            'feature': entry_feature,
            'description': default_description
        })
        self.cmd('appconfig feature set --connection-string {connection_string} --feature {feature} -y',
                 checks=[self.check('locked', default_locked),
                         self.check('name', entry_feature),
                         self.check('key', internal_feature_key),
                         self.check('description', default_description),
                         self.check('state', default_state),
                         self.check('conditions', default_conditions)])

        # Get information about account logged in with 'az login'
        appconfig_id = self.cmd('appconfig show -n {config_store_name} -g {rg}').get_output_in_json()['id']
        account_info = self.cmd('account show').get_output_in_json()
        endpoint = "https://" + config_store_name + ".azconfig.io"
        self.kwargs.update({
            'appconfig_id': appconfig_id,
            'user_id': account_info['user']['name'],
            'endpoint': endpoint
        })

        # Before assigning data reader role, read operation should fail with AAD auth.
        # The exception really depends on the which identity is used to run this testcase.
        with self.assertRaisesRegex(CLIError, "Operation returned an invalid status 'Forbidden'"):
            self.cmd('appconfig kv show --endpoint {endpoint} --auth-mode login --key {key}')

        # Assign data reader role to current user
        self.cmd('role assignment create --assignee {user_id} --role "App Configuration Data Reader" --scope {appconfig_id}')
        time.sleep(900)  # It takes around 15 mins for RBAC permissions to propagate

        # After asssigning data reader role, read operation should succeed
        self.cmd('appconfig kv show --endpoint {endpoint} --auth-mode login --key {key}',
                 checks=[self.check('key', entry_key),
                         self.check('value', entry_value)])

        # Since the logged in account also has "Contributor" role, providing --name instead of --endpoint should succeed
        self.cmd('appconfig feature show --name {config_store_name} --auth-mode login --feature {feature}',
                 checks=[self.check('locked', default_locked),
                         self.check('name', entry_feature),
                         self.check('key', internal_feature_key),
                         self.check('description', default_description),
                         self.check('state', default_state),
                         self.check('conditions', default_conditions)])

        # Write operations should fail with "Forbidden" error
        updated_value = "Blue"
        self.kwargs.update({
            'value': updated_value
        })
        with self.assertRaisesRegex(CLIError, "Operation returned an invalid status 'Forbidden'"):
            self.cmd('appconfig kv set --endpoint {endpoint} --auth-mode login --key {key} --value {value} -y')

        # Export from appconfig to file should succeed
        os.environ['AZURE_APPCONFIG_FM_COMPATIBLE'] = 'True'
        exported_file_path = os.path.join(TEST_DIR, 'export_aad_1.json')
        expected_exported_file_path = os.path.join(TEST_DIR, 'expected_export_aad_1.json')
        self.kwargs.update({
            'import_source': 'file',
            'imported_format': 'json',
            'separator': '/',
            'exported_file_path': exported_file_path
        })
        self.cmd(
            'appconfig kv export --endpoint {endpoint} --auth-mode login -d {import_source} --path "{exported_file_path}" --format {imported_format} --separator {separator} -y')
        with open(expected_exported_file_path) as json_file:
            expected_exported_kvs = json.load(json_file)
        with open(exported_file_path) as json_file:
            exported_kvs = json.load(json_file)
        assert expected_exported_kvs == exported_kvs
        os.remove(exported_file_path)

        # Assign data owner role to current user
        self.cmd('role assignment create --assignee {user_id} --role "App Configuration Data Owner" --scope {appconfig_id}')
        time.sleep(900)  # It takes around 15 mins for RBAC permissions to propagate

        # After assigning data owner role, write operation should succeed
        self.cmd('appconfig kv set --endpoint {endpoint} --auth-mode login --key {key} --value {value} -y',
                 checks=[self.check('key', entry_key),
                         self.check('value', updated_value)])

        # Add a KeyVault reference
        keyvault_key = "HostSecrets"
        keyvault_id = "https://fake.vault.azure.net/secrets/fakesecret"
        appconfig_keyvault_value = f"{{{json.dumps({'uri': keyvault_id})}}}"
        self.kwargs.update({
            'key': keyvault_key,
            'secret_identifier': keyvault_id
        })
        self.cmd('appconfig kv set-keyvault --endpoint {endpoint} --auth-mode login --key {key} --secret-identifier {secret_identifier} -y',
                 checks=[self.check('contentType', KeyVaultConstants.KEYVAULT_CONTENT_TYPE),
                         self.check('key', keyvault_key),
                         self.check('value', appconfig_keyvault_value)])

        # Import to appconfig should succeed
        imported_file_path = os.path.join(TEST_DIR, 'import_aad.json')
        exported_file_path = os.path.join(TEST_DIR, 'export_aad_2.json')
        expected_exported_file_path = os.path.join(TEST_DIR, 'expected_export_aad_2.json')
        self.kwargs.update({
            'imported_file_path': imported_file_path,
            'exported_file_path': exported_file_path
        })
        self.cmd(
            'appconfig kv import --endpoint {endpoint} --auth-mode login -s {import_source} --path "{imported_file_path}" --format {imported_format} --separator {separator} -y')

        # Export from appconfig to file should succeed
        self.cmd(
            'appconfig kv export --endpoint {endpoint} --auth-mode login -d {import_source} --path "{exported_file_path}" --format {imported_format} --separator {separator} -y')
        with open(expected_exported_file_path) as json_file:
            expected_exported_kvs = json.load(json_file)
        with open(exported_file_path) as json_file:
            exported_kvs = json.load(json_file)
        assert expected_exported_kvs == exported_kvs
        os.remove(exported_file_path)

    @unittest.skip("Incorrect test case")
    @mock.patch('azure.cli.core.auth.adal_authentication.MSIAuthenticationWrapper.set_token')
    @mock.patch('azure.cli.core.auth.adal_authentication.MSIAuthenticationWrapper.get_token')
    @mock.patch('azure.cli.core._profile.Profile.get_subscription')
    def test_azconfig_mi_token_override(self, get_subscriptions_mock, msi_get_token_mock, msi_set_token_mock):
        get_subscriptions_mock.return_value = TEST_MI_SUBSCRIPTION
        msi_get_token_mock.return_value = { "token": "mocked_token", "expires_on": 1234567890 }

        mock_cmd = mock.MagicMock()
        profile = Profile(cli_ctx=mock_cmd.cli_ctx)
        cred, _, _ = profile.get_login_credentials()

        appconfig_credential = AppConfigurationCliCredential(cred, APPCONFIG_AUTH_TOKEN_AUDIENCE)
        
        self.assertIsInstance(appconfig_credential._impl, MSIAuthenticationWrapper)
        self.assertTrue(hasattr(appconfig_credential._impl, 'get_token'))
        
        # Call get_token with an arbitrary scope
        _ = appconfig_credential.get_token("some_scope")

        # Assert that get_token was called with the correct scope
        appconfig_credential._impl.get_token.assert_called_once_with(f"{APPCONFIG_AUTH_TOKEN_AUDIENCE}/.default")

    @unittest.skip("Incorrect test case")
    @mock.patch('azure.cli.core.auth.msal_credentials.UserCredential')
    @mock.patch('azure.cli.core.auth.credential_adaptor.CredentialAdaptor.get_token')
    @mock.patch('azure.cli.core._profile.Profile.get_subscription')
    def test_azconfig_credential_adaptor_token_override(self, get_subscription_mock, get_token_mock, user_cred_mock):
        get_subscription_mock.return_value = TEST_USER_SUBSCRIPTION
        get_token_mock.return_value = { "token": "mocked_token", "expires_on": 1234567890 }

        profile = Profile(cli_ctx=DummyCli())
        cred, _, _ = profile.get_login_credentials()

        appconfig_credential = AppConfigurationCliCredential(cred, APPCONFIG_AUTH_TOKEN_AUDIENCE)
        
        self.assertIsInstance(appconfig_credential._impl, CredentialAdaptor)
        self.assertTrue(hasattr(appconfig_credential._impl, 'get_token'))
        
        # Call get_token with an arbitrary scope
        _ = appconfig_credential.get_token("some_scope")

        # Assert that get_token was called with the correct scope
        appconfig_credential._impl.get_token.assert_called_once_with(f"{APPCONFIG_AUTH_TOKEN_AUDIENCE}/.default")


    @live_only()
    @AllowLargeResponse()
    @ResourceGroupPreparer(parameter_name_for_location='location')
    def test_azconfig_user_token_audience(self, resource_group, location):
        aad_store_prefix = get_resource_name_prefix('AADStore')
        config_store_name = self.create_random_name(prefix=aad_store_prefix, length=24)

        location = 'eastus'
        sku = 'standard'
        entry_key = "Color"
        entry_value = "Red"

        self.kwargs.update({
            'config_store_name': config_store_name,
            'rg_loc': location,
            'rg': resource_group,
            'sku': sku
        })
        create_config_store(self, self.kwargs)
        
        appconfig_id = self.cmd('appconfig show -n {config_store_name} -g {rg}').get_output_in_json()['id']
        account_info = self.cmd('account show').get_output_in_json()
        endpoint = "https://" + config_store_name + ".azconfig.io"
        self.kwargs.update({
            'appconfig_id': appconfig_id,
            'user_id': account_info['user']['name'],
            'endpoint': endpoint
        })

        # Assign data reader role to current user
        self.cmd('role assignment create --assignee {user_id} --role "App Configuration Data Reader" --scope {appconfig_id}')

        self.kwargs.update({
            'entry_key': entry_key,
            'entry_value': entry_value
        })
        
        with mock.patch('azure.cli.command_modules.appconfig._credential.AppConfigurationCliCredential', wraps=AppConfigurationCliCredential) as cred_mock:
            try:
                self.cmd('appconfig kv set --endpoint {endpoint} --auth-mode login --key {entry_key} --value {entry_value} -y',
                        checks=[self.check('key', entry_key),
                                self.check('value', entry_value)])
            
            # Might return 403 forbidden error if the role assignment is not propagated yet
            # This is expected behavior, so we can ignore it
            except CLIError as e:
                if "Operation returned an invalid status 'Forbidden'" not in str(e):
                    raise e

            # Assert that the ClientCredential was instantiated with no custom scope
            cred_mock.assert_called_with(mock.ANY, None)

            # Mock the get_active_cloud function to return a custom cloud with a custom token audience
            with mock.patch('azure.cli.core.cloud.get_active_cloud', new=mock_get_active_cloud):
                try:
                    self.cmd('appconfig kv set --endpoint {endpoint} --auth-mode login --key {entry_key} --value {entry_value} -y',
                            checks=[self.check('key', entry_key),
                                    self.check('value', entry_value)])
            
                except CLIError as e:
                    if "Operation returned an invalid status 'Forbidden'" not in str(e):
                        raise e

                # Assert that the ClientCredential was instantiated with the correct scope
                cred_mock.assert_called_with(mock.ANY, APPCONFIG_AUTH_TOKEN_AUDIENCE)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "monitor metrics alert update",
)
class Update(AAZCommand):
    """Update a metric-based alert rule.

    :example: Disable/Enable a metric-based alert rule.
        az monitor metrics alert update --enabled false --name MyAlertRule --resource-group MyResourceGroup
    """

    _aaz_info = {
        "version": "2018-03-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.insights/metricalerts/{}", "2018-03-01"],
        ]
    }

    AZ_SUPPORT_GENERIC_UPDATE = True

    def _handler(self, command_args):
        super()._handler(command_args)
        self._execute_operations()
        return self._output()

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )
        _args_schema.name = AAZStrArg(
            options=["-n", "--name"],
            help="Name of the alert rule.",
            required=True,
            id_part="name",
        )
        _args_schema.auto_mitigate = AAZBoolArg(
            options=["--auto-mitigate"],
            help="Automatically resolve the alert.",
            nullable=True,
        )
        _args_schema.description = AAZStrArg(
            options=["--description"],
            help="Free-text description of the rule.",
            nullable=True,
        )
        _args_schema.enabled = AAZBoolArg(
            options=["--enabled"],
            help="Whether the metric alert rule is enabled.",
        )
        _args_schema.evaluation_frequency = AAZDurationArg(
            options=["--evaluation-frequency"],
            help="Frequency with which to evaluate the rule in `##h##m##s` format.",
        )
        _args_schema.scopes = AAZListArg(
            options=["--scopes"],
            help="Space-separated list of scopes the rule applies to. The resources specified in this parameter must be of the same type and exist in the same location.",
        )
        _args_schema.severity = AAZIntArg(
            options=["--severity"],
            help="Severity of the alert from 0 (critical) to 4 (verbose).",
        )
        _args_schema.window_size = AAZDurationArg(
            options=["--window-size"],
            help="Time over which to aggregate metrics in `##h##m##s` format.",
        )
        _args_schema.tags = AAZDictArg(
            options=["--tags"],
            help="Space-separated tags: key[=value] [key[=value] ...].",
            nullable=True,
        )

        scopes = cls._args_schema.scopes
        scopes.Element = AAZStrArg(
            nullable=True,
        )

        tags = cls._args_schema.tags
        tags.Element = AAZStrArg(
            nullable=True,
        )

        # define Arg Group "Properties"
        return cls._args_schema

    _args_metric_dimension_update = None

    @classmethod
    def _build_args_metric_dimension_update(cls, _schema):
        if cls._args_metric_dimension_update is not None:
            _schema.name = cls._args_metric_dimension_update.name
            _schema.operator = cls._args_metric_dimension_update.operator
            _schema.values = cls._args_metric_dimension_update.values
            return

        cls._args_metric_dimension_update = AAZObjectArg(
            nullable=True,
        )

        metric_dimension_update = cls._args_metric_dimension_update
        metric_dimension_update.name = AAZStrArg(
            options=["name"],
            help="Name of the dimension.",
        )
        metric_dimension_update.operator = AAZStrArg(
            options=["operator"],
            help="the dimension operator. Only 'Include' and 'Exclude' are supported",
        )
        metric_dimension_update.values = AAZListArg(
            options=["values"],
            help="list of dimension values.",
        )

        values = cls._args_metric_dimension_update.values
        values.Element = AAZStrArg(
            nullable=True,
        )

        _schema.name = cls._args_metric_dimension_update.name
        _schema.operator = cls._args_metric_dimension_update.operator
        _schema.values = cls._args_metric_dimension_update.values

    def _execute_operations(self):
        self.pre_operations()
        self.MetricAlertsGet(ctx=self.ctx)()
        self.pre_instance_update(self.ctx.vars.instance)
        self.InstanceUpdateByJson(ctx=self.ctx)()
        self.InstanceUpdateByGeneric(ctx=self.ctx)()
        self.post_instance_update(self.ctx.vars.instance)
        self.MetricAlertsCreateOrUpdate(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    @register_callback
    def pre_instance_update(self, instance):
        pass

    @register_callback
    def post_instance_update(self, instance):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class MetricAlertsGet(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Insights/metricAlerts/{ruleName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "GET"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "ruleName", self.ctx.args.name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2018-03-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()
            _UpdateHelper._build_schema_metric_alert_resource_read(cls._schema_on_200)

            return cls._schema_on_200

    class MetricAlertsCreateOrUpdate(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Insights/metricAlerts/{ruleName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "PUT"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "ruleName", self.ctx.args.name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2018-03-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Content-Type", "application/json",
                ),
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        @property
        def content(self):
            _content_value, _builder = self.new_content_builder(
                self.ctx.args,
                value=self.ctx.vars.instance,
            )

            return self.serialize_content(_content_value)

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()
            _UpdateHelper._build_schema_metric_alert_resource_read(cls._schema_on_200)

            return cls._schema_on_200

    class InstanceUpdateByJson(AAZJsonInstanceUpdateOperation):

        def __call__(self, *args, **kwargs):
            self._update_instance(self.ctx.vars.instance)

        def _update_instance(self, instance):
            _instance_value, _builder = self.new_content_builder(
                self.ctx.args,
                value=instance,
                typ=AAZObjectType
            )
            _builder.set_prop("properties", AAZObjectType, ".", typ_kwargs={"flags": {"required": True, "client_flatten": True}})
            _builder.set_prop("tags", AAZDictType, ".tags")

            properties = _builder.get(".properties")
            if properties is not None:
                properties.set_prop("autoMitigate", AAZBoolType, ".auto_mitigate")
                properties.set_prop("description", AAZStrType, ".description")
                properties.set_prop("enabled", AAZBoolType, ".enabled", typ_kwargs={"flags": {"required": True}})
                properties.set_prop("evaluationFrequency", AAZStrType, ".evaluation_frequency", typ_kwargs={"flags": {"required": True}})
                properties.set_prop("scopes", AAZListType, ".scopes", typ_kwargs={"flags": {"required": True}})
                properties.set_prop("severity", AAZIntType, ".severity", typ_kwargs={"flags": {"required": True}})
                properties.set_prop("windowSize", AAZStrType, ".window_size", typ_kwargs={"flags": {"required": True}})

            scopes = _builder.get(".properties.scopes")
            if scopes is not None:
                scopes.set_elements(AAZStrType, ".")

            tags = _builder.get(".tags")
            if tags is not None:
                tags.set_elements(AAZStrType, ".")

            return _instance_value

    class InstanceUpdateByGeneric(AAZGenericInstanceUpdateOperation):

        def __call__(self, *args, **kwargs):
            self._update_instance_by_generic(
                self.ctx.vars.instance,
                self.ctx.generic_update_args
            )


class _UpdateHelper:
    """Helper class for Update"""

    @classmethod
    def _build_schema_metric_dimension_update(cls, _builder):
        if _builder is None:
            return
        _builder.set_prop("name", AAZStrType, ".name", typ_kwargs={"flags": {"required": True}})
        _builder.set_prop("operator", AAZStrType, ".operator", typ_kwargs={"flags": {"required": True}})
        _builder.set_prop("values", AAZListType, ".values", typ_kwargs={"flags": {"required": True}})

        values = _builder.get(".values")
        if values is not None:
            values.set_elements(AAZStrType, ".")

    _schema_metric_alert_resource_read = None

    @classmethod
    def _build_schema_metric_alert_resource_read(cls, _schema):
        if cls._schema_metric_alert_resource_read is not None:
            _schema.id = cls._schema_metric_alert_resource_read.id
            _schema.location = cls._schema_metric_alert_resource_read.location
            _schema.name = cls._schema_metric_alert_resource_read.name
            _schema.properties = cls._schema_metric_alert_resource_read.properties
            _schema.tags = cls._schema_metric_alert_resource_read.tags
            _schema.type = cls._schema_metric_alert_resource_read.type
            return

        cls._schema_metric_alert_resource_read = _schema_metric_alert_resource_read = AAZObjectType()

        metric_alert_resource_read = _schema_metric_alert_resource_read
        metric_alert_resource_read.id = AAZStrType(
            flags={"read_only": True},
        )
        metric_alert_resource_read.location = AAZStrType(
            flags={"required": True},
        )
        metric_alert_resource_read.name = AAZStrType(
            flags={"read_only": True},
        )
        metric_alert_resource_read.properties = AAZObjectType(
            flags={"required": True, "client_flatten": True},
        )
        metric_alert_resource_read.tags = AAZDictType()
        metric_alert_resource_read.type = AAZStrType(
            flags={"read_only": True},
        )

        properties = _schema_metric_alert_resource_read.properties
        properties.actions = AAZListType()
        properties.auto_mitigate = AAZBoolType(
            serialized_name="autoMitigate",
        )
        properties.criteria = AAZObjectType(
            flags={"required": True},
        )
        properties.description = AAZStrType()
        properties.enabled = AAZBoolType(
            flags={"required": True},
        )
        properties.evaluation_frequency = AAZStrType(
            serialized_name="evaluationFrequency",
            flags={"required": True},
        )
        properties.is_migrated = AAZBoolType(
            serialized_name="isMigrated",
            flags={"read_only": True},
        )
        properties.last_updated_time = AAZStrType(
            serialized_name="lastUpdatedTime",
            flags={"read_only": True},
        )
        properties.scopes = AAZListType(
            flags={"required": True},
        )
        properties.severity = AAZIntType(
            flags={"required": True},
        )
        properties.target_resource_region = AAZStrType(
            serialized_name="targetResourceRegion",
        )
        properties.target_resource_type = AAZStrType(
            serialized_name="targetResourceType",
        )
        properties.window_size = AAZStrType(
            serialized_name="windowSize",
            flags={"required": True},
        )

        actions = _schema_metric_alert_resource_read.properties.actions
        actions.Element = AAZObjectType()

        _element = _schema_metric_alert_resource_read.properties.actions.Element
        _element.action_group_id = AAZStrType(
            serialized_name="actionGroupId",
        )
        _element.web_hook_properties = AAZDictType(
            serialized_name="webHookProperties",
        )

        web_hook_properties = _schema_metric_alert_resource_read.properties.actions.Element.web_hook_properties
        web_hook_properties.Element = AAZStrType()

        criteria = _schema_metric_alert_resource_read.properties.criteria
        criteria["odata.type"] = AAZStrType(
            flags={"required": True},
        )

        disc_microsoft__azure__monitor__multiple_resource_multiple_metric_criteria = _schema_metric_alert_resource_read.properties.criteria.discriminate_by("odata.type", "Microsoft.Azure.Monitor.MultipleResourceMultipleMetricCriteria")
        disc_microsoft__azure__monitor__multiple_resource_multiple_metric_criteria.all_of = AAZListType(
            serialized_name="allOf",
        )

        all_of = _schema_metric_alert_resource_read.properties.criteria.discriminate_by("odata.type", "Microsoft.Azure.Monitor.MultipleResourceMultipleMetricCriteria").all_of
        all_of.Element = AAZObjectType()

        _element = _schema_metric_alert_resource_read.properties.criteria.discriminate_by("odata.type", "Microsoft.Azure.Monitor.MultipleResourceMultipleMetricCriteria").all_of.Element
        _element.criterion_type = AAZStrType(
            serialized_name="criterionType",
            flags={"required": True},
        )
        _element.dimensions = AAZListType()
        _element.metric_name = AAZStrType(
            serialized_name="metricName",
            flags={"required": True},
        )
        _element.metric_namespace = AAZStrType(
            serialized_name="metricNamespace",
        )
        _element.name = AAZStrType(
            flags={"required": True},
        )
        _element.skip_metric_validation = AAZBoolType(
            serialized_name="skipMetricValidation",
        )
        _element.time_aggregation = AAZStrType(
            serialized_name="timeAggregation",
            flags={"required": True},
        )

        dimensions = _schema_metric_alert_resource_read.properties.criteria.discriminate_by("odata.type", "Microsoft.Azure.Monitor.MultipleResourceMultipleMetricCriteria").all_of.Element.dimensions
        dimensions.Element = AAZObjectType()
        cls._build_schema_metric_dimension_read(dimensions.Element)

        disc_dynamic_threshold_criterion = _schema_metric_alert_resource_read.properties.criteria.discriminate_by("odata.type", "Microsoft.Azure.Monitor.MultipleResourceMultipleMetricCriteria").all_of.Element.discriminate_by("criterion_type", "DynamicThresholdCriterion")
        disc_dynamic_threshold_criterion.alert_sensitivity = AAZStrType(
            serialized_name="alertSensitivity",
            flags={"required": True},
        )
        disc_dynamic_threshold_criterion.failing_periods = AAZObjectType(
            serialized_name="failingPeriods",
            flags={"required": True},
        )
        disc_dynamic_threshold_criterion.ignore_data_before = AAZStrType(
            serialized_name="ignoreDataBefore",
        )
        disc_dynamic_threshold_criterion.operator = AAZStrType(
            flags={"required": True},
        )

        failing_periods = _schema_metric_alert_resource_read.properties.criteria.discriminate_by("odata.type", "Microsoft.Azure.Monitor.MultipleResourceMultipleMetricCriteria").all_of.Element.discriminate_by("criterion_type", "DynamicThresholdCriterion").failing_periods
        failing_periods.min_failing_periods_to_alert = AAZFloatType(
            serialized_name="minFailingPeriodsToAlert",
            flags={"required": True},
        )
        failing_periods.number_of_evaluation_periods = AAZFloatType(
            serialized_name="numberOfEvaluationPeriods",
            flags={"required": True},
        )

        disc_static_threshold_criterion = _schema_metric_alert_resource_read.properties.criteria.discriminate_by("odata.type", "Microsoft.Azure.Monitor.MultipleResourceMultipleMetricCriteria").all_of.Element.discriminate_by("criterion_type", "StaticThresholdCriterion")
        disc_static_threshold_criterion.operator = AAZStrType(
            flags={"required": True},
        )
        disc_static_threshold_criterion.threshold = AAZFloatType(
            flags={"required": True},
        )

        disc_microsoft__azure__monitor__single_resource_multiple_metric_criteria = _schema_metric_alert_resource_read.properties.criteria.discriminate_by("odata.type", "Microsoft.Azure.Monitor.SingleResourceMultipleMetricCriteria")
        disc_microsoft__azure__monitor__single_resource_multiple_metric_criteria.all_of = AAZListType(
            serialized_name="allOf",
        )

        all_of = _schema_metric_alert_resource_read.properties.criteria.discriminate_by("odata.type", "Microsoft.Azure.Monitor.SingleResourceMultipleMetricCriteria").all_of
        all_of.Element = AAZObjectType()

        _element = _schema_metric_alert_resource_read.properties.criteria.discriminate_by("odata.type", "Microsoft.Azure.Monitor.SingleResourceMultipleMetricCriteria").all_of.Element
        _element.criterion_type = AAZStrType(
            serialized_name="criterionType",
            flags={"required": True},
        )
        _element.dimensions = AAZListType()
        _element.metric_name = AAZStrType(
            serialized_name="metricName",
            flags={"required": True},
        )
        _element.metric_namespace = AAZStrType(
            serialized_name="metricNamespace",
        )
        _element.name = AAZStrType(
            flags={"required": True},
        )
        _element.operator = AAZStrType(
            flags={"required": True},
        )
        _element.skip_metric_validation = AAZBoolType(
            serialized_name="skipMetricValidation",
        )
        _element.threshold = AAZFloatType(
            flags={"required": True},
        )
        _element.time_aggregation = AAZStrType(
            serialized_name="timeAggregation",
            flags={"required": True},
        )

        dimensions = _schema_metric_alert_resource_read.properties.criteria.discriminate_by("odata.type", "Microsoft.Azure.Monitor.SingleResourceMultipleMetricCriteria").all_of.Element.dimensions
        dimensions.Element = AAZObjectType()
        cls._build_schema_metric_dimension_read(dimensions.Element)

        disc_microsoft__azure__monitor__webtest_location_availability_criteria = _schema_metric_alert_resource_read.properties.criteria.discriminate_by("odata.type", "Microsoft.Azure.Monitor.WebtestLocationAvailabilityCriteria")
        disc_microsoft__azure__monitor__webtest_location_availability_criteria.component_id = AAZStrType(
            serialized_name="componentId",
            flags={"required": True},
        )
        disc_microsoft__azure__monitor__webtest_location_availability_criteria.failed_location_count = AAZFloatType(
            serialized_name="failedLocationCount",
            flags={"required": True},
        )
        disc_microsoft__azure__monitor__webtest_location_availability_criteria.web_test_id = AAZStrType(
            serialized_name="webTestId",
            flags={"required": True},
        )

        scopes = _schema_metric_alert_resource_read.properties.scopes
        scopes.Element = AAZStrType()

        tags = _schema_metric_alert_resource_read.tags
        tags.Element = AAZStrType()

        _schema.id = cls._schema_metric_alert_resource_read.id
        _schema.location = cls._schema_metric_alert_resource_read.location
        _schema.name = cls._schema_metric_alert_resource_read.name
        _schema.properties = cls._schema_metric_alert_resource_read.properties
        _schema.tags = cls._schema_metric_alert_resource_read.tags
        _schema.type = cls._schema_metric_alert_resource_read.type

    _schema_metric_dimension_read = None

    @classmethod
    def _build_schema_metric_dimension_read(cls, _schema):
        if cls._schema_metric_dimension_read is not None:
            _schema.name = cls._schema_metric_dimension_read.name
            _schema.operator = cls._schema_metric_dimension_read.operator
            _schema.values = cls._schema_metric_dimension_read.values
            return

        cls._schema_metric_dimension_read = _schema_metric_dimension_read = AAZObjectType()

        metric_dimension_read = _schema_metric_dimension_read
        metric_dimension_read.name = AAZStrType(
            flags={"required": True},
        )
        metric_dimension_read.operator = AAZStrType(
            flags={"required": True},
        )
        metric_dimension_read.values = AAZListType(
            flags={"required": True},
        )

        values = _schema_metric_dimension_read.values
        values.Element = AAZStrType()

        _schema.name = cls._schema_metric_dimension_read.name
        _schema.operator = cls._schema_metric_dimension_read.operator
        _schema.values = cls._schema_metric_dimension_read.values


__all__ = ["Update"]
